import logging

from actions.models import Navigation, NavigationProduct
from application.repository import NavigationRepository
from actions import mb_context
from typing import List


class NavigationInteractor(object):
    def __init__(self):
        # type: (int) -> None

        self.logger = logging.getLogger("PosActions")
        self.navigation = NavigationRepository(mb_context)

    def get_navigation_list(self):
        # type: () -> List[Navigation]

        try:
            return self.navigation.get_navigation_list()
        except Exception as _:
            self.logger.exception("Error getting navigation list")
            return []

    def get_navigation_products(self):
        # type: () -> List[NavigationProduct]

        try:
            return self.navigation.get_navigation_products()
        except Exception as _:
            self.logger.exception("Error getting navigation products")
            return []

    def get_navigations_restricted_to_service_period(self, service_period_id):
        # type: (str) -> List[str]

        try:
            return self.navigation.get_navigations_restricted_to_service_period(service_period_id)
        except Exception as _:
            self.logger.exception("Error getting navigation restricted to service period")
            return []



# encoding: utf-8


class JustificationBodyDto(object):
    def __init__(
        self,
        order_id,                       # type: str
        operator,                       # type: str
        authorized_by,                  # type: str
        discount_description,           # type: str
        justification,                  # type: str
    ):
        # type: (...) -> None

        self.order_id = order_id
        self.operator = operator
        self.authorized_by = authorized_by
        self.discount_description = discount_description
        self.justification = justification



from abc import ABCMeta, abstractmethod


class SubscriberCreator(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def create_subscriber(self, subject):
        # type: (str) -> int
        raise NotImplementedError()



import base64
from logging import Logger
from threading import Lock

from model.error_type import ExternalServicesErrorsType
from typing import Any

from interactor.route.order.finalize.dto import (
    OrderFinalizeRequest,
    FiscalOrderResponse,
)
from model.config import Config
from model.exception import ExternalServiceErrors
from model.repository import PosRepositoryModel
from validation_util import (
    check_if_business_day_is_opened,
    check_if_operator_is_logged,
    check_if_order_is_totaled,
    check_if_due_amount_is_zero,
    validate_api_key,
)

order_finalize_lock = {}


class OrderFinalizeInteractor(object):
    def __init__(self, logger, config, pos_repository, fiscal_service):
        # type: (Logger, Config, PosRepositoryModel, Any) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.fiscal_service = fiscal_service

    def execute(
        self,
        request,    # type: OrderFinalizeRequest
    ):
        # type: (...) -> FiscalOrderResponse

        pos_id = validate_api_key(request, self.config.api_key_mapping)
        with self._get_pos_lock(pos_id):
            model = self.pos_repository.get_model(pos_id)

            check_if_business_day_is_opened(self.pos_repository, model)
            check_if_operator_is_logged(self.pos_repository, model)
            check_if_order_is_totaled(self.pos_repository, model)
            check_if_due_amount_is_zero(model)

            pos_ot = self.pos_repository.get_pos_ot(model)
            try:
                response = self.fiscal_service.process_order_fiscal(
                    model=model,
                    pos_ot=pos_ot,
                    print_coupon=request.print_coupon,
                )
            except Exception as _:
                self.pos_repository.void_tenders_and_order(pos_id, pos_ot)
                self.logger.exception("Error fiscalizing order. The order has been voided")
                raise ExternalServiceErrors(ExternalServicesErrorsType.FiscalizeUnavailable)
            self.pos_repository.set_order_to_paid(model, pos_ot)

            return FiscalOrderResponse(
                fiscal_coupon=base64.b64encode(response),
            )

    @staticmethod
    def _get_pos_lock(
            pos_id,     # type: str
    ):
        # type: (...) -> Lock

        global order_finalize_lock

        if pos_id not in order_finalize_lock:
            order_finalize_lock[pos_id] = Lock()

        return order_finalize_lock[pos_id]



from xml.etree import cElementTree as eTree

from application.interactor import OrderModifiedPaidInteractor
from application.model import AsyncUuidMessageProcessor
from application.repository.pos.utils import get_originator_pos_id
from messagebus import DefaultToken, DataType, Message, MessageBus, Event
from typing import Optional, Any


class OrderModifiedPaidProcessor(AsyncUuidMessageProcessor):
    
    def __init__(self, interactor):
        # type: (OrderModifiedPaidInteractor) -> None

        super(OrderModifiedPaidProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data):
        # type: (str) -> (int, int, int, eTree)

        order_picture = eTree.XML(data).find(".//Order")
        current_pos_id = int(order_picture.get("posId"))
        originator_pos_id = get_originator_pos_id(order_picture) or current_pos_id
        order_id = int(order_picture.get("orderId"))

        return current_pos_id, originator_pos_id, order_id, order_picture

    def call_business(self, params):
        # type: ((int, int, int, eTree)) -> None

        current_pos_id, originator_pos_id, order_id, order_picture = params
        return self.interactor.burn_paid_benefit(current_pos_id, originator_pos_id, order_id, order_picture)

    def format_response(self, message_bus, message, event, request, response):
        # type: (MessageBus, Optional[Message], Optional[Event], Any, Any) -> None
        return

    def format_exception(self, message_bus, message, event, input_obj, exception):
        # type: (MessageBus, Optional[Message], Optional[Event], Any, BaseException) -> None
        return

    def format_parse_exception(self, message_bus, message, event, data, exception):
        # type: (MessageBus, Optional[Message], Optional[Event], bytes, BaseException) -> None
        return



import math
from unittest import TestCase
from simplejson.compat import long_type, text_type
import simplejson as json
from simplejson.decoder import NaN, PosInf, NegInf

class TestFloat(TestCase):
    def test_degenerates_allow(self):
        for inf in (PosInf, NegInf):
            self.assertEqual(json.loads(json.dumps(inf)), inf)
        # Python 2.5 doesn't have math.isnan
        nan = json.loads(json.dumps(NaN))
        self.assertTrue((0 + nan) != nan)

    def test_degenerates_ignore(self):
        for f in (PosInf, NegInf, NaN):
            self.assertEqual(json.loads(json.dumps(f, ignore_nan=True)), None)

    def test_degenerates_deny(self):
        for f in (PosInf, NegInf, NaN):
            self.assertRaises(ValueError, json.dumps, f, allow_nan=False)

    def test_floats(self):
        for num in [1617161771.7650001, math.pi, math.pi**100,
                    math.pi**-100, 3.1]:
            self.assertEqual(float(json.dumps(num)), num)
            self.assertEqual(json.loads(json.dumps(num)), num)
            self.assertEqual(json.loads(text_type(json.dumps(num))), num)

    def test_ints(self):
        for num in [1, long_type(1), 1<<32, 1<<64]:
            self.assertEqual(json.dumps(num), str(num))
            self.assertEqual(int(json.dumps(num)), num)
            self.assertEqual(json.loads(json.dumps(num)), num)
            self.assertEqual(json.loads(text_type(json.dumps(num))), num)



import warnings
import six
import sys
import qrcode
import qrcode.util
import qrcode.image.svg

try:
    import qrcode.image.pure
    import pymaging_png  # ensure that PNG support is installed
except ImportError:  # pragma: no cover
    pymaging_png = None

import qrcode
from qrcode.image.base import BaseImage
from qrcode.exceptions import DataOverflowError
from qrcode.util import (
    QRData, MODE_NUMBER, MODE_ALPHA_NUM, MODE_8BIT_BYTE)
from qrcode.tests.svg import SvgImageWhite

try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    from unittest import mock
except ImportError:
    import mock

UNICODE_TEXT = u'\u03b1\u03b2\u03b3'


class QRCodeTests(unittest.TestCase):

    def test_basic(self):
        qr = qrcode.QRCode(version=1)
        qr.add_data('a')
        qr.make(fit=False)

    def test_large(self):
        qr = qrcode.QRCode(version=27)
        qr.add_data('a')
        qr.make(fit=False)

    def test_invalid_version(self):
        qr = qrcode.QRCode(version=41)
        self.assertRaises(ValueError, qr.make, fit=False)

    def test_overflow(self):
        qr = qrcode.QRCode(version=1)
        qr.add_data('abcdefghijklmno')
        self.assertRaises(DataOverflowError, qr.make, fit=False)

    def test_add_qrdata(self):
        qr = qrcode.QRCode(version=1)
        data = QRData('a')
        qr.add_data(data)
        qr.make(fit=False)

    def test_fit(self):
        qr = qrcode.QRCode()
        qr.add_data('a')
        qr.make()
        self.assertEqual(qr.version, 1)
        qr.add_data('bcdefghijklmno')
        qr.make()
        self.assertEqual(qr.version, 2)

    def test_mode_number(self):
        qr = qrcode.QRCode()
        qr.add_data('1234567890123456789012345678901234', optimize=0)
        qr.make()
        self.assertEqual(qr.version, 1)
        self.assertEqual(qr.data_list[0].mode, MODE_NUMBER)

    def test_mode_alpha(self):
        qr = qrcode.QRCode()
        qr.add_data('ABCDEFGHIJ1234567890', optimize=0)
        qr.make()
        self.assertEqual(qr.version, 1)
        self.assertEqual(qr.data_list[0].mode, MODE_ALPHA_NUM)

    def test_regression_mode_comma(self):
        qr = qrcode.QRCode()
        qr.add_data(',', optimize=0)
        qr.make()
        self.assertEqual(qr.data_list[0].mode, MODE_8BIT_BYTE)

    def test_mode_8bit(self):
        qr = qrcode.QRCode()
        qr.add_data(u'abcABC' + UNICODE_TEXT, optimize=0)
        qr.make()
        self.assertEqual(qr.version, 1)
        self.assertEqual(qr.data_list[0].mode, MODE_8BIT_BYTE)

    def test_mode_8bit_newline(self):
        qr = qrcode.QRCode()
        qr.add_data('ABCDEFGHIJ1234567890\n', optimize=0)
        qr.make()
        self.assertEqual(qr.data_list[0].mode, MODE_8BIT_BYTE)

    def test_render_pil(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image()
        img.save(six.BytesIO())

    def test_qrcode_bad_factory(self):
        self.assertRaises(
            TypeError, qrcode.QRCode, image_factory='not_BaseImage')
        self.assertRaises(
            AssertionError, qrcode.QRCode, image_factory=dict)

    def test_qrcode_factory(self):

        class MockFactory(BaseImage):
            drawrect = mock.Mock()

        qr = qrcode.QRCode(image_factory=MockFactory)
        qr.add_data(UNICODE_TEXT)
        qr.make_image()
        self.assertTrue(MockFactory.drawrect.called)

    def test_render_svg(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=qrcode.image.svg.SvgImage)
        img.save(six.BytesIO())

    def test_render_svg_path(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=qrcode.image.svg.SvgPathImage)
        img.save(six.BytesIO())

    def test_render_svg_fragment(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=qrcode.image.svg.SvgFragmentImage)
        img.save(six.BytesIO())

    def test_render_svg_with_background(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=SvgImageWhite)
        img.save(six.BytesIO())

    @unittest.skipIf(not pymaging_png, "Requires pymaging with PNG support")
    def test_render_pymaging_png(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=qrcode.image.pure.PymagingImage)
        with warnings.catch_warnings():
            if six.PY3:
                warnings.simplefilter('ignore', DeprecationWarning)
            img.save(six.BytesIO())

    @unittest.skipIf(not pymaging_png, "Requires pymaging")
    def test_render_pymaging_png_bad_kind(self):
        qr = qrcode.QRCode()
        qr.add_data(UNICODE_TEXT)
        img = qr.make_image(image_factory=qrcode.image.pure.PymagingImage)
        self.assertRaises(ValueError, img.save, six.BytesIO(), kind='FISH')

    def test_optimize(self):
        qr = qrcode.QRCode()
        text = 'A1abc12345def1HELLOa'
        qr.add_data(text, optimize=4)
        qr.make()
        self.assertEqual(
            [d.mode for d in qr.data_list],
            [
                MODE_8BIT_BYTE, MODE_NUMBER, MODE_8BIT_BYTE, MODE_ALPHA_NUM,
                MODE_8BIT_BYTE
            ]
        )
        self.assertEqual(qr.version, 2)

    def test_optimize_short(self):
        qr = qrcode.QRCode()
        text = 'A1abc1234567def1HELLOa'
        qr.add_data(text, optimize=7)
        qr.make()
        self.assertEqual(len(qr.data_list), 3)
        self.assertEqual(
            [d.mode for d in qr.data_list],
            [MODE_8BIT_BYTE, MODE_NUMBER, MODE_8BIT_BYTE]
        )
        self.assertEqual(qr.version, 2)

    def test_optimize_longer_than_data(self):
        qr = qrcode.QRCode()
        text = 'ABCDEFGHIJK'
        qr.add_data(text, optimize=12)
        self.assertEqual(len(qr.data_list), 1)
        self.assertEqual(qr.data_list[0].mode, MODE_ALPHA_NUM)

    def test_optimize_size(self):
        text = 'A1abc12345123451234512345def1HELLOHELLOHELLOHELLOa' * 5

        qr = qrcode.QRCode()
        qr.add_data(text)
        qr.make()
        self.assertEqual(qr.version, 10)

        qr = qrcode.QRCode()
        qr.add_data(text, optimize=0)
        qr.make()
        self.assertEqual(qr.version, 11)

    def test_qrdata_repr(self):
        data = b'hello'
        data_obj = qrcode.util.QRData(data)
        self.assertEqual(repr(data_obj), repr(data))

    def test_print_ascii_stdout(self):
        qr = qrcode.QRCode()
        stdout_encoding = sys.stdout.encoding
        with mock.patch('sys.stdout') as fake_stdout:
            # Python 2.6 needs sys.stdout.encoding to be a real string.
            sys.stdout.encoding = stdout_encoding
            fake_stdout.isatty.return_value = None
            self.assertRaises(OSError, qr.print_ascii, tty=True)
            self.assertTrue(fake_stdout.isatty.called)

    def test_print_ascii(self):
        qr = qrcode.QRCode(border=0)
        f = six.StringIO()
        qr.print_ascii(out=f)
        printed = f.getvalue()
        f.close()
        expected = u'\u2588\u2580\u2580\u2580\u2580\u2580\u2588'
        self.assertEqual(printed[:len(expected)], expected)

        f = six.StringIO()
        f.isatty = lambda: True
        qr.print_ascii(out=f, tty=True)
        printed = f.getvalue()
        f.close()
        expected = (
            u'\x1b[48;5;232m\x1b[38;5;255m' +
            u'\xa0\u2584\u2584\u2584\u2584\u2584\xa0')
        self.assertEqual(printed[:len(expected)], expected)

    def test_print_tty_stdout(self):
        qr = qrcode.QRCode()
        with mock.patch('sys.stdout') as fake_stdout:
            fake_stdout.isatty.return_value = None
            self.assertRaises(OSError, qr.print_tty)
            self.assertTrue(fake_stdout.isatty.called)

    def test_print_tty(self):
        qr = qrcode.QRCode()
        f = six.StringIO()
        f.isatty = lambda: True
        qr.print_tty(out=f)
        printed = f.getvalue()
        f.close()
        BOLD_WHITE_BG = '\x1b[1;47m'
        BLACK_BG = '\x1b[40m'
        WHITE_BLOCK = BOLD_WHITE_BG + '  ' + BLACK_BG
        EOL = '\x1b[0m\n'
        expected = (
            BOLD_WHITE_BG + '  '*23 + EOL +
            WHITE_BLOCK + '  '*7 + WHITE_BLOCK)
        self.assertEqual(printed[:len(expected)], expected)

    def test_get_matrix(self):
        qr = qrcode.QRCode(border=0)
        qr.add_data('1')
        self.assertEqual(qr.get_matrix(), qr.modules)

    def test_get_matrix_border(self):
        qr = qrcode.QRCode(border=1)
        qr.add_data('1')
        matrix = [row[1:-1] for row in qr.get_matrix()[1:-1]]
        self.assertEqual(matrix, qr.modules)

    def test_negative_size_at_construction(self):
        self.assertRaises(ValueError, qrcode.QRCode, box_size=-1)

    def test_negative_size_at_usage(self):
        qr = qrcode.QRCode()
        qr.box_size = -1
        self.assertRaises(ValueError, qr.make_image)


class ShortcutTest(unittest.TestCase):

    def runTest(self):
        qrcode.make('image')



# ASN.1 "character string" types
from pyasn1.type import univ, tag

class NumericString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 18)
        )

class PrintableString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 19)
        )

class TeletexString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 20)
        )

class T61String(TeletexString): pass

class VideotexString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 21)
        )

class IA5String(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 22)
        )

class GraphicString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 25)
        )

class VisibleString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 26)
        )

class ISO646String(VisibleString): pass

class GeneralString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 27)
        )

class UniversalString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 28)
        )
    encoding = "utf-32-be"

class BMPString(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 30)
        )
    encoding = "utf-16-be"

class UTF8String(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 12)
        )
    encoding = "utf-8"



'''
Custom exceptions raised by pytz.
'''

__all__ = [
    'UnknownTimeZoneError', 'InvalidTimeError', 'AmbiguousTimeError',
    'NonExistentTimeError',
]


class UnknownTimeZoneError(KeyError):
    '''Exception raised when pytz is passed an unknown timezone.

    >>> isinstance(UnknownTimeZoneError(), LookupError)
    True

    This class is actually a subclass of KeyError to provide backwards
    compatibility with code relying on the undocumented behavior of earlier
    pytz releases.

    >>> isinstance(UnknownTimeZoneError(), KeyError)
    True
    '''
    pass


class InvalidTimeError(Exception):
    '''Base class for invalid time exceptions.'''


class AmbiguousTimeError(InvalidTimeError):
    '''Exception raised when attempting to create an ambiguous wallclock time.

    At the end of a DST transition period, a particular wallclock time will
    occur twice (once before the clocks are set back, once after). Both
    possibilities may be correct, unless further information is supplied.

    See DstTzInfo.normalize() for more info
    '''


class NonExistentTimeError(InvalidTimeError):
    '''Exception raised when attempting to create a wallclock time that
    cannot exist.

    At the start of a DST transition period, the wallclock time jumps forward.
    The instants jumped over never occur.
    '''



from abc import ABCMeta, abstractmethod

from datetime import datetime  # noqa


class Clock(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_current_date(self):
        # type: () -> datetime
        raise NotImplementedError()

    @abstractmethod
    def get_current_datetime(self):
        # type: () -> datetime
        raise NotImplementedError()



import json
import logging

from logging import Logger

from bustoken import TK_NCOUPON_TEST_CONNECTION
from msgbus import FM_PARAM, TK_SYS_ACK
from sysactions import send_message


class NCouponInteractor(object):
    def __init__(self):
        # type: () -> None

        self.logger = logging.getLogger("PosActions")   # type: Logger

    def test_connection(
        self,
        pos_id,     # type: bool
    ):
        # type: (...) -> bool

        try:
            data = json.dumps({
                "posId": pos_id
            })
            response = send_message("nCoupon", TK_NCOUPON_TEST_CONNECTION, FM_PARAM, data, timeout=30000000)
            return response.token == TK_SYS_ACK
        except Exception as _:
            self.logger.exception("Error testing nCoupon connection")
            return False



from ._GetLoyaltyIdProcessor import GetLoyaltyIdProcessor
from ._GetAndLockVoucherProcessor import GetAndLockVoucherProcessor
from ._BurnVoucherProcessor import BurnVoucherProcessor
from ._UnlockVoucherProcessor import UnlockVoucherProcessor
from ._LoyaltyAvailableProcessor import LoyaltyAvailableProcessor



from helper import ExtendedEnum


class PosValidationErrorsType(ExtendedEnum):
    # 403 messages (ValidationError)
    PosIdNotFound = "PosId not found"
    OrderIsNotInProgress = "Order is not in progress"
    OrderAlreadyInProgress = "There is already an order registered in that PosId"
    OrderIsNotTotaled = "Order is not totaled"
    OrderIsNotInProgressOrTotaled = "Order is not in progress or totaled"
    OperatorIsNotLogged = "Operator is not logged"
    BusinessDayNotOpened = "Business day is not opened"
    PaymentAlreadyExists = "Impossible release shopping cart due to payment made"
    AlreadyHasTender = "Order already has tender"
    OrderNotFound = "Order not found or already canceled"
    CurrentOrderNotFound = "No current order available"
    PaymentAmountInvalid = "Payment amount is invalid"
    PaymentTypeInvalid = "Payment type is invalid"
    KeyPropertyInvalid = "This key property is invalid"
    DueAmountInvalid = "Invalid due amount"
    SaleTypeInvalid = "Sale type not available or invalid"
    ParamsNotFound = "Params not found"
    ScannerNotFound = "Peripheral scanner not found"
    PrinterNotFound = "Peripheral printer not found"
    InvalidProduct = "Invalid product"
    NoCachedCoupons = "No coupons cached, please reload the cache"
    TimeoutApiCoupon = "Time out api coupon"
    CouponInvalid = "Invalid Coupon"
    ApiCouponInvalid = "Invalid API Coupon"
    ExpiredCoupon = "Expired Coupon"
    CouponWeekDayInvalid = "Invalid Coupon for this weekday"
    PosProductUnavailable = "Product not available in this POS"
    AlreadyUsedCoupon = "Coupon has already been used"
    CannotApplyCoupon = "{}"
    SaleItemsInvalid = "Received items invalid or nonexistent"
    ItemTypeComboInvalid = "Cannot multiply an item with 'combo' type"
    InvalidQuantity = "Invalid quantity received"
    FiscalCertificateError = "Error on fiscal certificate"
    EmptyPrinterContent = "Printert content must not be empty"
    InvalidOrderId = "Invalid orderId"
    LanguagesNotFound = "Languages not found"
    PaymentTypesNotFound = "Payment types not found"
    PaymentTypesToShowNotFound = "Without valid Payment Types to show"
    EmptyStoreConfigurationForPos = "Empty store configuration for current POS"
    InvalidTenderId = "Invalid Tender ID"
    ProductInfoNotFound = "There is no product info for this language"
    PropertyRequiredNotFound = "Property {} is required"
    InvalidWorkingMode = "Invalid pos working mode"


class AuthErrorsType(ExtendedEnum):
    # 401 messages (AuthValidation)
    InvalidApiKey = "Invalid API key"


class OperationErrorsType(ExtendedEnum):
    # 500 messages (OperationErrors)
    OperatorLogoutError = "Unable to logout operator"
    OperatorLoginError = "Unable to login operator"
    BusynessDayCloseError = "Unable to close business period"
    BusynessDayOpenError = "Unable to open business period"
    FiscalModifierError = "Error processing fiscal modifier"
    InvalidPaymentResponse = "Invalid response from service"
    UnexpectedPaymentResponse = "Unexpected payment response"
    InvalidPaymentService = "Has no electronic payment service configured for this POS"


class ExternalServicesErrorsType(ExtendedEnum):

    # 503 messages (ExternalServiceErrors)
    MenuDataNotAvailable = "Can't obtain menu data"
    MenuNavigationNotAvailable = "Can not obtain menu navigation"
    UnableToGetStatus = "Unable to get status"
    UnableToGetMenuId = "Unable to get menu id"
    InsertPaymentUnavailable = "Unable to insert payment"
    FiscalizeUnavailable = "Unable to fiscalize order"
    ServiceUnavailable = "Service Temporary Unavailable"
    ElectronicMessageUnavailable = "Cannot send electronic messages"
    ErrorPrinting = "Error printing text to printer: {}"
    LanguageValuesNotFound = "Text for language '{}' not found"
    CannotGetStoreConfigForPos = "Cannot get store config for POS {}"
    CannotGetNavigationForPos = "Cannot get navigation menu for POS {}"
    StoredOrderError = "Error on stored order"


class BadRequestErrorsType(ExtendedEnum):
    # 400 messages (BadRequestErrors)
    InvalidPosId = "Invalid PosId"


class NotFoundErrorsType(ExtendedEnum):
    # 504 messages (# 503 messages (ExternalServiceErrors))
    CustomerNotFound = "Customer not found"



from messagebus import TokenCreator, TokenPriority


class NDiscountTokens(object):
    NDISCOUNT_MESSAGE_GROUP = "112"

    TK_NDISCOUNT_ADD_BENEFIT = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "1")
    TK_NDISCOUNT_REMOVE = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "2")
    TK_NDISCOUNT_APPLY_AUTOMATIC_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "6")
    TK_NDISCOUNT_GET_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "7")


class LoyaltyTokens(object):
    LOYALTY_MESSAGE_GROUP = "111"

    TK_LOYALTY_GET_AND_LOCK_VOUCHER = TokenCreator.create_token(TokenPriority.low, LOYALTY_MESSAGE_GROUP, "2")
    TK_LOYALTY_BURN_VOUCHER = TokenCreator.create_token(TokenPriority.low, LOYALTY_MESSAGE_GROUP, "3")
    TK_LOYALTY_UNLOCK_VOUCHER = TokenCreator.create_token(TokenPriority.low, LOYALTY_MESSAGE_GROUP, "4")



from abc import ABCMeta, abstractmethod

from ._MessageProcessor import MessageProcessor
from ._MessageProcessorExecutor import MessageProcessorExecutor


class MessageProcessorExecutorFactory(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def build_executor(self, message_processor):
        # type: (MessageProcessor) -> MessageProcessorExecutor
        raise NotImplementedError()



def contexts_match(
    item_context,       # type: str
    part_code_context,  # type: str
):
    # type: (...) -> bool

    if part_code_context == "1":
        return True if part_code_context == item_context else False

    item_context = item_context.split('.')
    part_code_context = part_code_context.split('.')
    item_context = [x for x in item_context if x not in ['', '1']]
    part_code_context = [x for x in part_code_context if x not in ['', '1']]
    context_result = []
    for part_code_slice in part_code_context:
        for idx, item_slice in enumerate(item_context):
            if item_slice == part_code_slice:
                item_context = item_context[idx:]
                context_result.append(item_slice)
                break

    return len(part_code_context) == len(context_result)



from _I18n import I18n

from datetime import datetime
import iso8601
from application.repository import I18nRepository

from typing import List  # noqa


class I18nImpl(I18n):
    def __init__(self, i18n_repository):
        # type: (I18nRepository) -> None
        self.i18n_repository = i18n_repository

    def internationalize(self, labels):
        labels_to_translate = self.get_labels_to_translate(labels)
        labels_to_localize = self.get_labels_to_localize(labels)
        translations = self.i18n_repository.internationalize(labels_to_translate)
        localizations = self.localize_labels(labels_to_localize)
        return self.create_internationalized_dict(labels, localizations, translations)

    def internationalize_text(self, text):
        labels = []
        for label in text.split(" "):
            words = self.break_same_words(label)
            labels.extend(words)

        translations = self.internationalize(labels)
        final_text = text
        for word in translations:
            final_text = final_text.replace(word, translations[word])
        return final_text

    def break_same_words(self, label):
        # type: (str) -> List[str]
        index = label.find("$")
        if index < 0:
            return self.break_functions(label)

        first_word = []
        if index > 0:
            first_word = self.break_functions(label[0:index])

        other_word_index = label.find("$", index + 1)
        if other_word_index < 0:
            return first_word + self.break_functions(label[index:])
        else:
            return first_word + self.break_functions(label[index:other_word_index]) + self.break_same_words(
                label[other_word_index:])

    def break_functions(self, label):
        if len(label) == 0:
            return []

        index = label.find("#")
        if index < 0:
            return [label]

        first_word = []
        if index > 0:
            first_word = [label[0:index]]

        other_word_index = label.find(")", index + 1)
        return first_word + [label[index:other_word_index + 1]] + self.break_functions(label[other_word_index + 1:])

    @staticmethod
    def get_labels_to_translate(labels):
        labels_to_translate = []
        for word in labels:
            if word.startswith("$"):
                labels_to_translate.append(word)
        return labels_to_translate

    @staticmethod
    def get_labels_to_localize(labels):
        labels_to_localize = []
        for word in labels:
            if word.startswith("#"):
                labels_to_localize.append(word)
        return labels_to_localize

    def localize_labels(self, labels_to_localize):
        localizations = {}
        l10n_processor = L10nProcessor(self.i18n_repository)
        for label in labels_to_localize:
            localizations[label] = l10n_processor.process_l10n_function(label)
        return localizations

    @staticmethod
    def create_internationalized_dict(labels, localizations, translations):
        translated_labels = {}
        for word in labels:
            if word not in translated_labels:
                I18nImpl.add_word_to_translated_labels(word, translated_labels, translations, localizations)
        return translated_labels

    @staticmethod
    def add_word_to_translated_labels(word, translated_labels, translations, localizations):
        if word in translations:
            translated_labels[word] = translations[word]
        elif word in localizations:
            translated_labels[word] = localizations[word]
        else:
            translated_labels[word] = word


class L10nProcessor(object):
    def __init__(self, i18n_repository):
        # type: (I18nRepository) -> None
        self.i18n_repository = i18n_repository

        self.processors = {
            "DATETIME": self.process_date_time,
            "DATE": self.process_date,
            "CURRENCY_SYMBOL": self.process_currency_symbol,
            "CURRENCY": self.process_currency,
            "NUMBER": self.process_number,
        }

    def process_l10n_function(self, l10n_func):
        func_name = L10nProcessor.get_func_name(l10n_func)
        param_value = L10nProcessor.get_param_value(l10n_func)

        processor = self.processors.get(func_name)
        if processor is None:
            return l10n_func
        else:
            return processor(param_value)

    @staticmethod
    def get_func_name(l10n_function):
        index = l10n_function.index("(")
        return l10n_function[1:index]

    @staticmethod
    def get_param_value(l10n_function):
        index = l10n_function.index("(")
        index2 = l10n_function.index(")")
        return l10n_function[index + 1:index2]

    def process_date_time(self, value):
        return datetime.strftime(iso8601.parse_date(value), self.i18n_repository.get_date_time_format())

    def process_date(self, value):
        return datetime.strftime(iso8601.parse_date(value), self.i18n_repository.get_date_format())

    def process_currency_symbol(self, _):
        return self.i18n_repository.get_currency_symbol()

    def process_currency(self, value):
        currency_symbol_position = self.i18n_repository.get_currency_symbol_position()
        if currency_symbol_position == "after":
            return self.process_number(value) + " " + self.i18n_repository.get_currency_symbol()
        elif currency_symbol_position == "after_no_space":
            return self.process_number(value) + self.i18n_repository.get_currency_symbol()
        elif currency_symbol_position == "before_no_space":
            return self.i18n_repository.get_currency_symbol() + self.process_number(value)
        else:
            return self.i18n_repository.get_currency_symbol() + " " + self.process_number(value)

    def process_number(self, value):
        int_value = int(float(value))
        int_part = "{:,d}".format(int_value).replace(",", self.i18n_repository.get_thousands_separator())
        decimal_part = ("{:." + str(self.i18n_repository.get_decimal_places()) + "f}").format(float(value) - int_value) \
                           .replace(".", self.i18n_repository.get_decimal_separator())[1:]
        return int_part + decimal_part



from logging import Logger

from helper import json_serialize
from model.error_type import PosValidationErrorsType
from interactor.route.order.add_items.dto import AddItemsRequest
from model.config import Config
from model.exception import ValidationError
from model.repository import PosRepositoryModel
from validation_util import \
    check_if_business_day_is_opened, \
    check_if_operator_is_logged, \
    check_if_order_is_in_progress, validate_api_key


class AddItemsInteractor(object):

    def __init__(self, logger, config, pos_repository):
        # type: (Logger, Config, PosRepositoryModel) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository

    def execute(self, request):
        # type: (AddItemsRequest) -> None

        pos_id = validate_api_key(request, self.config.api_key_mapping)
        self.logger.info("Adding items to order. PosId: {}; Items: {}"
                         .format(pos_id, json_serialize(request.items)))

        if not request.items:
            raise ValidationError(PosValidationErrorsType.SaleItemsInvalid)

        model = self.pos_repository.get_model(pos_id)

        check_if_business_day_is_opened(self.pos_repository, model)
        check_if_operator_is_logged(self.pos_repository, model)
        check_if_order_is_in_progress(self.pos_repository, model)

        pos_ot = self.pos_repository.get_pos_ot(model)
        price_list = self.pos_repository.get_price_list(model)

        for item in request.items:
            self.logger.info("Adding item. Context: {}, LineNumber: {}, ProductCode: {}; Quantity: {}".format(
                item.context,
                item.line_number,
                item.product_code,
                item.quantity
            ))
            self.pos_repository.add_item_to_order(
                pos_id=pos_id,
                pos_ot=pos_ot,
                price_list=price_list,
                item=item
            )

        pos_ot.blkopnotify = False
        pos_ot.updateOrderProperties(pos_id)



# coding: utf-8
from __future__ import unicode_literals, division, absolute_import, print_function

import inspect
import sys


if sys.version_info < (3,):
    str_cls = unicode  # noqa
    byte_cls = str
    int_types = (int, long)  # noqa

    def bytes_to_list(byte_string):
        return [ord(b) for b in byte_string]

    chr_cls = chr

else:
    str_cls = str
    byte_cls = bytes
    int_types = int

    bytes_to_list = list

    def chr_cls(num):
        return bytes([num])


def type_name(value):
    """
    Returns a user-readable name for the type of an object

    :param value:
        A value to get the type name of

    :return:
        A unicode string of the object's type name
    """

    if inspect.isclass(value):
        cls = value
    else:
        cls = value.__class__
    if cls.__module__ in set(['builtins', '__builtin__']):
        return cls.__name__
    return '%s.%s' % (cls.__module__, cls.__name__)



from typing import Union

from reportsdata.justificationreport import (
    JustificationReportGenerator,
    JustificationReportPresenter,
)


class ReportGenerator(object):

    @staticmethod
    def generate(
        mb_context,     # type: MBEasyContext
        pos_id,         # type: str
        *args           # type: Union[str, unicode]
    ):
        # type: (...) -> tuple

        generator = JustificationReportGenerator(
            pos_id=pos_id,
            data=args[0],
        )

        presenter = JustificationReportPresenter(
            columns=38,
        )

        return generator, presenter



from domain import Clock

from datetime import datetime


class RealClock(Clock):
    def get_current_date(self):
        return datetime.utcnow().date()

    def get_current_datetime(self):
        return datetime.utcnow()



# -*- coding: utf-8 -*-
import sysactions
from application.repository import AccountRepository
from actions.util import get_operator_info, get_user_level
from ..functions.do_transfer import do_transfer, get_justification
from .. import logger, SIGNED_IN_USER, mb_context
from .utils import close_user

USECS_PER_SEC = 1000000


def common_close_user(pos_id, model, user_id, authorizer=None):
    _, _, session = get_operator_info(model, user_id)
    pod_function = sysactions.get_posfunction(model)
    is_quick_service = model.find("WorkingMode").get("usrCtrlType") != "TS"
    need_ask_justification = False

    if pod_function != "OT":
        response = do_transfer(pos_id, 6, user_id, is_quick_service, authorizer)
        done_transfer = response['done_transfer']
        need_ask_justification = response['need_ask_justification']
        _set_authorizer(pos_id, authorizer, is_quick_service)
        if not done_transfer:
            return False

    logger.debug("Logoff POS %d - Deslogando operador %s" % (int(pos_id), user_id))

    period = sysactions.get_business_period(model)
    if close_user(pos_id, user_id):
        _set_authorizer(
            pos_id=pos_id,
            authorizer=authorizer,
            is_quick_service=is_quick_service
        )
        logger.debug("Logoff POS %d - Operador %s deslogado com sucesso" % (int(pos_id), user_id))
        pos_ot = sysactions.get_posot(model)
        pos_ot.resetCurrentOrder(pos_id)
        sysactions.set_custom(pos_id, "VOIDAUTH_SESSION_DISABLED", "false", persist=True)
        sysactions.clear_custom(pos_id, SIGNED_IN_USER)
    else:
        logger.error("Logoff POS %d - Falha ao deslogar operador %s" % (int(pos_id), user_id))
        sysactions.show_messagebox(pos_id, "$OPERATION_FAILED", "$ERROR")
        return False

    report = "close_operator_report"
    sysactions.print_report(pos_id, model, True, report, pos_id, period, user_id, session, authorizer)

    if pod_function != "OT" and need_ask_justification:
        account_repository = AccountRepository(mb_context)
        get_justification(account_repository, pos_id, session)

    if need_ask_justification:
        sysactions.print_report(pos_id, model, True, report, pos_id, period, user_id, session, authorizer)

    return True


def _set_authorizer(pos_id, authorizer, is_quick_service):
    if authorizer is not None and is_quick_service is False:
        sysactions.set_custom(pos_id, 'Last Manager ID', authorizer)
        sysactions.set_custom(pos_id, 'Authorization Level', get_user_level(authorizer))



import iso8601

from deliveryreport.model import DeliveryOrder


class DeliveryHeaderDto(object):
    def __init__(
            self,
            order   # type: DeliveryOrder
    ):
        # type: (...) -> None

        self.store_name = order.store_name
        self.order_number = order.order_number
        self.order_date = iso8601.parse_date(order.order_date)
        self.partner = order.partner



from abc import ABCMeta, abstractmethod
from typing import Any


class CouponServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_prices(self, posid):
        # type: (int) -> Any

        raise NotImplementedError




# -*- coding: utf-8 -*-

import base64
import logging
import os
from xml.etree import cElementTree as eTree

import iso8601
from persistence import Driver

CHECKING_SENT_ORDERS_EXCEPTION_MESSAGE = "Error checking sent orders in database"


logger = logging.getLogger("Maintenance")


class RecreateXMLFilesProcessor(object):
    def __init__(self, mbcontext, comp_no, fiscal_xml_path, fiscal_sended):
        self.mbcontext = mbcontext
        self.comp_no = comp_no
        self.fiscal_xml_path = fiscal_xml_path
        self.fiscal_sended = fiscal_sended

        self.maintenance_processor = None

    def set_server_parameters(self, maintenance_processor):
        self.maintenance_processor = maintenance_processor

    def recreate_response_fiscal(self, recreate_days="30"):
        if self.comp_no != 0:
            return

        conn = None
        errors = []
        try:
            conn = Driver().open(self.mbcontext, service_name="FiscalPersistence")
            total_rows = int(self._get_count_rows_recreate_file(recreate_days) or 0)

            limit_start = 0
            while total_rows > 0:
                if (total_rows - 1000) < 0:
                    total_rows = 0
                else:
                    total_rows -= 1000

                cursor = conn.select("""SELECT PosId, OrderId, NumeroSat, date(datetime(datanota, 'unixepoch')) as data_nota, NumeroNota, XMLRequest, XMLResponse
                     FROM fiscal.FiscalData WHERE SentToBkOffice = 1 and date(datetime(datanota, 'unixepoch')) > date('now', '-{0} day') ORDER BY OrderId LIMIT 1000 OFFSET {1}
                        """.format(recreate_days, str(limit_start)))

                limit_start += 1000

                # Create directories and files
                for row in cursor:
                    try:
                        self._create_file(
                            conn,
                            errors=errors,
                            order_id=row.get_entry("OrderId").zfill(9),
                            pos_id=row.get_entry("PosId").zfill(2),
                            sat_num=row.get_entry("NumeroSat").zfill(2),
                            xml_request_data=row.get_entry("XMLRequest")
                        )

                    except Exception as ex:
                        errors.append("Error creating file: %s" % str(ex))
                        logger.exception("Error creating file")

        except Exception as ex:
            logger.exception(CHECKING_SENT_ORDERS_EXCEPTION_MESSAGE)
            return False, str(ex)
        finally:
            if conn:
                conn.close()
        if len(errors) > 0:
            return False, str(errors)
        return True, 'Files processed'

    def recreate_file_number(self, order_id):
        if self.comp_no != 0:
            return

        conn = None
        try:
            conn = Driver().open(self.mbcontext, service_name="FiscalPersistence")

            cursor = conn.select(
                """SELECT 
                PosId, OrderId, NumeroSat, date(datetime(datanota, 'unixepoch')) as data_nota, NumeroNota, XMLRequest, XMLResponse
                FROM fiscal.FiscalData WHERE orderid = '{}';""".format(order_id))
            if cursor.rows() > 0:
                for row in cursor:
                    self._create_file(
                        conn,
                        errors=[],
                        order_id=row.get_entry("OrderId").zfill(9),
                        pos_id=row.get_entry("PosId").zfill(2),
                        sat_num=row.get_entry("NumeroSat").zfill(2),
                        xml_request_data=row.get_entry("XMLRequest")
                    )
            else:
                return False, 'Missing number'
        except Exception as _:
            logger.exception(CHECKING_SENT_ORDERS_EXCEPTION_MESSAGE)
            return False, "Error processing XML"
        finally:
            if conn:
                conn.close()

        return True, 'File processed with sucess'

    def _get_count_rows_recreate_file(self, days_to_recreate):
        conn = None
        try:
            conn = Driver().open(self.mbcontext, service_name="FiscalPersistence")
            cursor = conn.select("""SELECT count(*) as total FROM FiscalData 
                                    WHERE SentToBkOffice = 1 
                                    and date(datetime(datanota, 'unixepoch')) > date('now', '-{0} day');"""
                                 .format(days_to_recreate))

            for row in cursor:
                return row.get_entry("total")
        except Exception as _:
            logger.exception(CHECKING_SENT_ORDERS_EXCEPTION_MESSAGE)
        finally:
            if conn:
                conn.close()

    def _create_file(self, conn, errors, order_id, pos_id, sat_num, xml_request_data):
        path_enviados = os.path.join(self.fiscal_xml_path, self.fiscal_sended)

        # Check if file exists
        xml_request = base64.b64decode(xml_request_data) or ''
        xml_request = self.maintenance_processor.remove_ns_from_xml(xml_text=xml_request)
        fiscal_type = None
        if xml_request.find("<NFe") != -1 and xml_request.find("<CFe") == -1:
            fiscal_type = "NFCe"
        if xml_request.find("<NFe") == -1 and xml_request.find("<CFe") != -1:
            fiscal_type = "SAT"

        if not fiscal_type:
            logger.exception("[_create_file] Erro fiscal_type = %s" % fiscal_type)

        save_path = None
        if fiscal_type in "NFCe":
            index1 = xml_request.index("<NFe")
            index2 = xml_request.index("</NFe>")
            nfce_xml = xml_request[index1:index2 + 6]
            nfce_xml = nfce_xml.replace("<Signature>", "<Signature xmlns=\"http://www.w3.org/2000/09/xmldsig#\">")

            parsed_xml = eTree.XML(nfce_xml)
            data_emissao_str = parsed_xml.find("infNFe/ide/dhEmi").text
            data_emissao = iso8601.parse_date(data_emissao_str)
            save_path = os.path.join(path_enviados, data_emissao.strftime("%Y"), data_emissao.strftime("%m"),
                                     data_emissao.strftime("%d"))
        if fiscal_type in "SAT":
            index1 = xml_request.index("<CFe")
            index2 = xml_request.index("</CFe>")
            nfce_xml = xml_request[index1:index2 + 6]
            nfce_xml = nfce_xml.replace("<Signature>", "<Signature xmlns=\"http://www.w3.org/2000/09/xmldsig#\">")

            parsed_xml = eTree.XML(nfce_xml)
            data_emissao_str = parsed_xml.find("infCFe/ide/dEmi").text
            data_emissao = iso8601.parse_date(data_emissao_str)
            save_path = os.path.join(path_enviados, data_emissao.strftime("%Y"), data_emissao.strftime("%m"),
                                     data_emissao.strftime("%d"))

        if not save_path:
            logger.exception("[_create_file] Erro save_path = {}".format(save_path))

        # Save files into bin
        xml_request = base64.b64decode(xml_request_data) or ''

        try:
            if fiscal_type in "SAT":
                self.maintenance_processor.save_file_from_sat_response(None, xml_request, order_id, pos_id, sat_num)
            if fiscal_type in "NFCe":
                self.maintenance_processor.save_file_from_nfce_request(None, xml_request, order_id, pos_id)

        except Exception as ex:
            logger.exception("Error creating order file: %s" % order_id)
            errors.append("Error creating order file: [%s, %s]" % (order_id, str(ex)))



from ast import literal_eval
from itertools import chain
from itertools import islice

from . import nodes
from ._compat import text_type
from .compiler import CodeGenerator
from .compiler import has_safe_repr
from .environment import Environment
from .environment import Template


def native_concat(nodes):
    """Return a native Python type from the list of compiled nodes. If
    the result is a single node, its value is returned. Otherwise, the
    nodes are concatenated as strings. If the result can be parsed with
    :func:`ast.literal_eval`, the parsed value is returned. Otherwise,
    the string is returned.

    :param nodes: Iterable of nodes to concatenate.
    """
    head = list(islice(nodes, 2))

    if not head:
        return None

    if len(head) == 1:
        raw = head[0]
    else:
        raw = u"".join([text_type(v) for v in chain(head, nodes)])

    try:
        return literal_eval(raw)
    except (ValueError, SyntaxError, MemoryError):
        return raw


class NativeCodeGenerator(CodeGenerator):
    """A code generator which renders Python types by not adding
    ``to_string()`` around output nodes.
    """

    @staticmethod
    def _default_finalize(value):
        return value

    def _output_const_repr(self, group):
        return repr(u"".join([text_type(v) for v in group]))

    def _output_child_to_const(self, node, frame, finalize):
        const = node.as_const(frame.eval_ctx)

        if not has_safe_repr(const):
            raise nodes.Impossible()

        if isinstance(node, nodes.TemplateData):
            return const

        return finalize.const(const)

    def _output_child_pre(self, node, frame, finalize):
        if finalize.src is not None:
            self.write(finalize.src)

    def _output_child_post(self, node, frame, finalize):
        if finalize.src is not None:
            self.write(")")


class NativeEnvironment(Environment):
    """An environment that renders templates to native Python types."""

    code_generator_class = NativeCodeGenerator


class NativeTemplate(Template):
    environment_class = NativeEnvironment

    def render(self, *args, **kwargs):
        """Render the template to produce a native Python type. If the
        result is a single node, its value is returned. Otherwise, the
        nodes are concatenated as strings. If the result can be parsed
        with :func:`ast.literal_eval`, the parsed value is returned.
        Otherwise, the string is returned.
        """
        vars = dict(*args, **kwargs)

        try:
            return native_concat(self.root_render_func(self.new_context(vars)))
        except Exception:
            return self.environment.handle_exception()


NativeEnvironment.template_class = NativeTemplate



from threading import Condition

from .. import Waiter


class PendingWaiter(Waiter):
    def __init__(self, sync_id):
        # type: (str) -> None
        self.sync_id = sync_id
        self.event = None
        self.condition = Condition()

    def get_sync_id(self):
        return self.sync_id

    def get_events(self):
        with self.condition:
            if self.event is None:
                self.condition.wait()
        return [self.event]

    def new_event(self, event):
        with self.condition:
            self.event = event
            self.condition.notify()



from logging import Logger
from typing import (
    Optional,
    List,
)

from interactor.route.payment.insert.dto import (
    OrderInsertPaymentRequest,
    OrderInsertPaymentResponse,
)
from model.config import (
    Config,
    MapValidatePosId,
)
from model.error_type import (
    ExternalServicesErrorsType,
)
from model.exception import (
    ExternalServiceErrors,
)
from model.repository import PosRepositoryModel
from validation_util import (
    check_if_business_day_is_opened,
    check_if_operator_is_logged,
    check_if_order_is_totaled,
    check_if_payment_type_is_valid,
    check_if_payment_amount_is_valid,
    validate_pos_id,
)


payment_types_cache = None


class OrderInsertPaymentInteractor(object):
    def __init__(
        self,
        logger,             # type: Logger
        config,             # type: Config
        pos_repository,     # type: PosRepositoryModel
        fiscal_service,     # type: FiscalService
        payment_service     # type: Optional[PaymentService]
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.fiscal_service = fiscal_service
        self.payment_service = payment_service

    def execute(
        self,
        request,     # type: OrderInsertPaymentRequest
    ):
        # type: (...) -> OrderInsertPaymentResponse

        model_validate = MapValidatePosId(
            pos_id=request.pos_id,
            system_api_key=request.system_api_key,
            secure_api_key_list=self.config.api_key_mapping,
        )

        pos_id = validate_pos_id(
            request=model_validate,
        )

        model = self.pos_repository.get_model(pos_id)
        pos_ot = self.pos_repository.get_pos_ot(
            model=model,
        )
        request = self._change_to_correct_electronic_type_id(request)
        check_if_business_day_is_opened(self.pos_repository, model)
        check_if_operator_is_logged(self.pos_repository, model)
        check_if_order_is_totaled(self.pos_repository, model)
        check_if_payment_type_is_valid(self.config.allowed_payment_methods, request.electronic_type_id)
        check_if_payment_amount_is_valid(request.amount, model)

        if request.is_register_payment:
            tender_details = self.payment_service.get_tender_details(
                payment_data=request,
            )
        else:
            try:
                tender_details = self.fiscal_service.process_payment(model, request)
            except Exception as _:
                self.logger.exception("Error performing payment")
                raise ExternalServiceErrors(ExternalServicesErrorsType.InsertPaymentUnavailable)

        tender = self.pos_repository.do_tender(
            pos_id=pos_id,
            tender_id=request.payment_id,
            pos_ot=pos_ot,
            amount=request.amount,
            tender_details=tender_details,
        )

        return OrderInsertPaymentResponse(tender)

    def _change_to_correct_electronic_type_id(
        self,
        request     # type: OrderInsertPaymentRequest
    ):
        # type: (...) -> OrderInsertPaymentRequest

        global payment_types_cache

        if not self.payment_service:
            return request

        if not payment_types_cache:
            payment_types = self.payment_service.get_payment_types()
            if not payment_types:
                return request
            payment_types_cache = payment_types

        request.electronic_type_id = self._filter_electronic_type_id(
            payment_types_cache,
            request.payment_id
        )

        return request

    @staticmethod
    def _filter_electronic_type_id(
        payment_types,   # type: List[PaymentType]
        default_value,   # type: int
    ):
        # type: (...) -> int

        return next(
            (item.electronic_type_id for item in payment_types if item.payment_type == default_value),
            default_value
        )



from domain import TransferRepository, Transfer
from old_helper import BaseRepository
from persistence import Connection

from typing import List  # noqa


class MwBusinessPeriodTransferRepository(TransferRepository, BaseRepository):
    def __init__(self, mbcontext):
        super(MwBusinessPeriodTransferRepository, self).__init__(mbcontext)

    def get_transfers(self, initial_date, end_date):
        def inner_func(conn):
            # type: (Connection) -> List[Transfer]
            query = self._TransfersByBusinessPeriod.format(initial_date.strftime("%Y%m%d"),
                                                           end_date.strftime("%Y%m%d"))

            return [Transfer(Transfer.Type(int(x.get_entry(0))), float(x.get_entry(1))) for x in conn.select(query)]

        return self.execute_with_connection(inner_func)

    _TransfersByBusinessPeriod = """
select Type, Amount
from Transfer where Period >= '{0}' and Period <= '{1}'"""



from ._Report import Report


class JsonReport(Report):
    def __init__(self, parts, width):
        self.parts = parts
        self.width = width

    def get_parts(self):
        return self.parts

    def get_width(self):
        return self.width



from helper import ExtendedEnum
from messagebus import (
    TokenCreator,
    TokenPriority,
)

MSGGRP_DEVICE_MANAGER = "D"


class ListenedTokens(ExtendedEnum):
    # Processors
    TK_GET_USED_SERVICE = TokenCreator.create_token(TokenPriority.low, MSGGRP_DEVICE_MANAGER, "1")



# -*- coding: utf-8 -*-
from xml.etree import cElementTree as eTree

import msgbus
import sysactions

from actions.models import UserLevels
from bustoken import TK_FPR_OK, TK_FPR_IDENTIFY_USER
from model.customexception import GettingUserInfoException, CanceledDialogException
from msgbus import MBMessage
from systools import sys_log_info, sys_log_warning
from typing import Dict, Optional
from utilfunctions import get_device_service

from .. import mb_context


def get_user_info(
        pos_id,         # type: str
        user_id=None,   # type: Optional[str]
        password=None,  # type: Optional[str]
        model=None      # type: eTree.XML
):
    # type: (...) -> Dict

    if model is None:
        model = sysactions.get_model(pos_id)

    user_got_from_finger_print = False
    if user_id is None:
        user_id = _get_username_from_finger_print_reader(pos_id)
        if user_id is not None:
            user_got_from_finger_print = True

    if user_id is None:
        user_id = _get_user_id_from_ui(pos_id, model)

    if not user_got_from_finger_print:
        if password is None:
            password = _get_user_password_from_ui(pos_id, model, user_id)

        sysactions.authenticate_user(user_id, password)

    user_xml_str = _get_user_xml(user_id)
    user_info = _build_user_info_from_user_xml(user_id, password, user_xml_str)
    return user_info


def _get_username_from_finger_print_reader(
        pos_id  # type: str
):
    # type: (...) -> Optional[str]

    try:
        if _is_finger_print_device_available(pos_id):
            finger_print_data = _get_finger_print_data(pos_id)
            if _was_finger_print_successfully_acquired(finger_print_data) or not _was_user_acquired(finger_print_data):
                sysactions.show_messagebox(pos_id, "$ERROR_READING_FINGERPRINT", "$ERROR")
                raise GettingUserInfoException()
            else:
                return finger_print_data.data

    except msgbus.MBException as ex:
        if ex.errorcode == 2:  # NOT_FOUND, FingerPrint not Available
            sys_log_info("FingerPrint Service Not Available")
        else:
            sys_log_warning("Error get_user_information - %s" % str(ex))

    return None


def _is_finger_print_device_available(
        pos_id  # type: str
):
    # type: (...) -> bool

    finger_print = get_device_service(
        pos_id=pos_id,
        device_type="fingerReader",
    )
    if not finger_print:
        return False

    ret = mb_context.MB_EasySendMessage("FingerPrintReader{0}".format(pos_id), TK_FPR_OK)
    return ret.token == msgbus.TK_SYS_ACK


def _get_user_id_from_ui(
        pos_id,     # type: str
        model       # type: eTree.XML
):
    # type: (...) -> int

    level_name = sysactions.translate_message(model, UserLevels.OPERATOR.name)
    display_title = "$USER_AUTHENTICATION_WITH_PARAMETER|{}".format(level_name)
    username = sysactions.show_keyboard(pos_id, "$ENTER_USER_ID", title=display_title, mask="INTEGER", numpad=True)
    if username in (None, ""):
        raise CanceledDialogException()

    return username


def _get_user_password_from_ui(
        pos_id,     # type: str
        model,      # type: eTree.XML
        user_id     # type: str
):
    # type: (...) -> int

    level_name = sysactions.translate_message(model, UserLevels.OPERATOR.name)
    display_title = "$USER_AUTHENTICATION_WITH_PARAMETER|{}".format(level_name)
    message = "$ENTER_PASSWORD|{}".format(user_id)
    password = sysactions.show_keyboard(pos_id, message, title=display_title, mask="PASSWORD", numpad=True)
    if password in (None, ""):
        raise CanceledDialogException()

    return password


def _get_user_xml(
        user_id     # type: str
):
    # type: (...) -> str

    try:
        user_xml_str = sysactions.get_user_information(user_id)
    except Exception as _:
        raise GettingUserInfoException()

    if user_xml_str is None:
        raise GettingUserInfoException()

    return user_xml_str


def _build_user_info_from_user_xml(
        user_id,        # type: str
        password,       # type: str
        user_xml_str    # type: str
):
    # type: (...) -> Dict

    user_xml = eTree.XML(user_xml_str)
    user_element = user_xml.find("user")
    return {
        'Level': user_element.attrib["Level"],
        'UserName': user_element.attrib["UserName"],
        'LongName': user_element.attrib["LongName"],
        'UserId': user_id,
        'Password': password
    }


def _get_finger_print_data(
        pos_id  # type: str
):
    # type: (...) -> MBMessage

    dest_name = "FingerPrintReader{0}".format(pos_id)
    return mb_context.MB_EasySendMessage(dest_name, TK_FPR_IDENTIFY_USER, format=msgbus.FM_PARAM, data=pos_id)


def _was_finger_print_successfully_acquired(
        finger_print_data   # type: MBMessage
):
    # type: (...) -> bool

    return finger_print_data.token != msgbus.TK_SYS_ACK


def _was_user_acquired(
        finger_print_data   # type: MBMessage
):
    # type: (...) -> bool

    return finger_print_data.data != ""



# -*- coding: utf-8 -*-

import os

import cfgtools
from messagehandler import MessageHandler
from msgbus import MBEasyContext
from helper import config_logger, import_pydevd

from ChatControllerEventHandler import ChatControllerEventHandler
from chatservice import ChatService, MessageParser
from chatservice.model import Event
from chatservice.repository import MessageRepository


def main():
    loader_cfg = os.environ["LOADERCFG"]
    service_name = "ChatController"
    required_services = "DeliveryPersistence"
    
    import_pydevd(loader_cfg, 9145)
    config_logger(loader_cfg, service_name)

    mb_context = MBEasyContext(service_name)
    message_handler = MessageHandler(mb_context, service_name, service_name, required_services, None)

    config = cfgtools.read(loader_cfg)
    sync_interval = int(config.find_value("ChatController.SyncInterval") or 10)
    
    message_repository = MessageRepository(mb_context)
    message_parser = MessageParser()
    
    chat_service = ChatService(mb_context, message_repository, message_parser, sync_interval)
    event_handler = ChatControllerEventHandler(mb_context, chat_service)

    message_handler.subscribe_reentrant_events([Event.SAC_CHAT_MESSAGE_NEW, Event.SAC_CHAT_MESSAGE_ACK])
    message_handler.set_event_handler(event_handler)

    message_handler.handle_events()


if __name__ == "__main__":
    main()



# coding: utf-8

"""
ASN.1 type classes for certificate revocation lists (CRL). Exports the
following items:

 - CertificateList()

Other type classes are defined that help compose the types listed above.
"""

from __future__ import unicode_literals, division, absolute_import, print_function

import hashlib

from .algos import SignedDigestAlgorithm
from .core import (
    Boolean,
    Enumerated,
    GeneralizedTime,
    Integer,
    ObjectIdentifier,
    OctetBitString,
    ParsableOctetString,
    Sequence,
    SequenceOf,
)
from .x509 import (
    AuthorityInfoAccessSyntax,
    AuthorityKeyIdentifier,
    CRLDistributionPoints,
    DistributionPointName,
    GeneralNames,
    Name,
    ReasonFlags,
    Time,
)


# The structures in this file are taken from https://tools.ietf.org/html/rfc5280


class Version(Integer):
    _map = {
        0: 'v1',
        1: 'v2',
        2: 'v3',
    }


class IssuingDistributionPoint(Sequence):
    _fields = [
        ('distribution_point', DistributionPointName, {'explicit': 0, 'optional': True}),
        ('only_contains_user_certs', Boolean, {'implicit': 1, 'default': False}),
        ('only_contains_ca_certs', Boolean, {'implicit': 2, 'default': False}),
        ('only_some_reasons', ReasonFlags, {'implicit': 3, 'optional': True}),
        ('indirect_crl', Boolean, {'implicit': 4, 'default': False}),
        ('only_contains_attribute_certs', Boolean, {'implicit': 5, 'default': False}),
    ]


class TBSCertListExtensionId(ObjectIdentifier):
    _map = {
        '2.5.29.18': 'issuer_alt_name',
        '2.5.29.20': 'crl_number',
        '2.5.29.27': 'delta_crl_indicator',
        '2.5.29.28': 'issuing_distribution_point',
        '2.5.29.35': 'authority_key_identifier',
        '2.5.29.46': 'freshest_crl',
        '1.3.6.1.5.5.7.1.1': 'authority_information_access',
    }


class TBSCertListExtension(Sequence):
    _fields = [
        ('extn_id', TBSCertListExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'issuer_alt_name': GeneralNames,
        'crl_number': Integer,
        'delta_crl_indicator': Integer,
        'issuing_distribution_point': IssuingDistributionPoint,
        'authority_key_identifier': AuthorityKeyIdentifier,
        'freshest_crl': CRLDistributionPoints,
        'authority_information_access': AuthorityInfoAccessSyntax,
    }


class TBSCertListExtensions(SequenceOf):
    _child_spec = TBSCertListExtension


class CRLReason(Enumerated):
    _map = {
        0: 'unspecified',
        1: 'key_compromise',
        2: 'ca_compromise',
        3: 'affiliation_changed',
        4: 'superseded',
        5: 'cessation_of_operation',
        6: 'certificate_hold',
        8: 'remove_from_crl',
        9: 'privilege_withdrawn',
        10: 'aa_compromise',
    }

    @property
    def human_friendly(self):
        """
        :return:
            A unicode string with revocation description that is suitable to
            show to end-users. Starts with a lower case letter and phrased in
            such a way that it makes sense after the phrase "because of" or
            "due to".
        """

        return {
            'unspecified': 'an unspecified reason',
            'key_compromise': 'a compromised key',
            'ca_compromise': 'the CA being compromised',
            'affiliation_changed': 'an affiliation change',
            'superseded': 'certificate supersession',
            'cessation_of_operation': 'a cessation of operation',
            'certificate_hold': 'a certificate hold',
            'remove_from_crl': 'removal from the CRL',
            'privilege_withdrawn': 'privilege withdrawl',
            'aa_compromise': 'the AA being compromised',
        }[self.native]


class CRLEntryExtensionId(ObjectIdentifier):
    _map = {
        '2.5.29.21': 'crl_reason',
        '2.5.29.23': 'hold_instruction_code',
        '2.5.29.24': 'invalidity_date',
        '2.5.29.29': 'certificate_issuer',
    }


class CRLEntryExtension(Sequence):
    _fields = [
        ('extn_id', CRLEntryExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'crl_reason': CRLReason,
        'hold_instruction_code': ObjectIdentifier,
        'invalidity_date': GeneralizedTime,
        'certificate_issuer': GeneralNames,
    }


class CRLEntryExtensions(SequenceOf):
    _child_spec = CRLEntryExtension


class RevokedCertificate(Sequence):
    _fields = [
        ('user_certificate', Integer),
        ('revocation_date', Time),
        ('crl_entry_extensions', CRLEntryExtensions, {'optional': True}),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _crl_reason_value = None
    _invalidity_date_value = None
    _certificate_issuer_value = None
    _issuer_name = False

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['crl_entry_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def crl_reason_value(self):
        """
        This extension indicates the reason that a certificate was revoked.

        :return:
            None or a CRLReason object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._crl_reason_value

    @property
    def invalidity_date_value(self):
        """
        This extension indicates the suspected date/time the private key was
        compromised or the certificate became invalid. This would usually be
        before the revocation date, which is when the CA processed the
        revocation.

        :return:
            None or a GeneralizedTime object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._invalidity_date_value

    @property
    def certificate_issuer_value(self):
        """
        This extension indicates the issuer of the certificate in question,
        and is used in indirect CRLs. CRL entries without this extension are
        for certificates issued from the last seen issuer.

        :return:
            None or an x509.GeneralNames object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._certificate_issuer_value

    @property
    def issuer_name(self):
        """
        :return:
            None, or an asn1crypto.x509.Name object for the issuer of the cert
        """

        if self._issuer_name is False:
            self._issuer_name = None
            if self.certificate_issuer_value:
                for general_name in self.certificate_issuer_value:
                    if general_name.name == 'directory_name':
                        self._issuer_name = general_name.chosen
                        break
        return self._issuer_name


class RevokedCertificates(SequenceOf):
    _child_spec = RevokedCertificate


class TbsCertList(Sequence):
    _fields = [
        ('version', Version, {'optional': True}),
        ('signature', SignedDigestAlgorithm),
        ('issuer', Name),
        ('this_update', Time),
        ('next_update', Time, {'optional': True}),
        ('revoked_certificates', RevokedCertificates, {'optional': True}),
        ('crl_extensions', TBSCertListExtensions, {'explicit': 0, 'optional': True}),
    ]


class CertificateList(Sequence):
    _fields = [
        ('tbs_cert_list', TbsCertList),
        ('signature_algorithm', SignedDigestAlgorithm),
        ('signature', OctetBitString),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _issuer_alt_name_value = None
    _crl_number_value = None
    _delta_crl_indicator_value = None
    _issuing_distribution_point_value = None
    _authority_key_identifier_value = None
    _freshest_crl_value = None
    _authority_information_access_value = None
    _issuer_cert_urls = None
    _delta_crl_distribution_points = None
    _sha1 = None
    _sha256 = None

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['tbs_cert_list']['crl_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def issuer_alt_name_value(self):
        """
        This extension allows associating one or more alternative names with
        the issuer of the CRL.

        :return:
            None or an x509.GeneralNames object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._issuer_alt_name_value

    @property
    def crl_number_value(self):
        """
        This extension adds a monotonically increasing number to the CRL and is
        used to distinguish different versions of the CRL.

        :return:
            None or an Integer object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._crl_number_value

    @property
    def delta_crl_indicator_value(self):
        """
        This extension indicates a CRL is a delta CRL, and contains the CRL
        number of the base CRL that it is a delta from.

        :return:
            None or an Integer object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._delta_crl_indicator_value

    @property
    def issuing_distribution_point_value(self):
        """
        This extension includes information about what types of revocations
        and certificates are part of the CRL.

        :return:
            None or an IssuingDistributionPoint object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._issuing_distribution_point_value

    @property
    def authority_key_identifier_value(self):
        """
        This extension helps in identifying the public key with which to
        validate the authenticity of the CRL.

        :return:
            None or an AuthorityKeyIdentifier object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._authority_key_identifier_value

    @property
    def freshest_crl_value(self):
        """
        This extension is used in complete CRLs to indicate where a delta CRL
        may be located.

        :return:
            None or a CRLDistributionPoints object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._freshest_crl_value

    @property
    def authority_information_access_value(self):
        """
        This extension is used to provide a URL with which to download the
        certificate used to sign this CRL.

        :return:
            None or an AuthorityInfoAccessSyntax object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._authority_information_access_value

    @property
    def issuer(self):
        """
        :return:
            An asn1crypto.x509.Name object for the issuer of the CRL
        """

        return self['tbs_cert_list']['issuer']

    @property
    def authority_key_identifier(self):
        """
        :return:
            None or a byte string of the key_identifier from the authority key
            identifier extension
        """

        if not self.authority_key_identifier_value:
            return None

        return self.authority_key_identifier_value['key_identifier'].native

    @property
    def issuer_cert_urls(self):
        """
        :return:
            A list of unicode strings that are URLs that should contain either
            an individual DER-encoded X.509 certificate, or a DER-encoded CMS
            message containing multiple certificates
        """

        if self._issuer_cert_urls is None:
            self._issuer_cert_urls = []
            if self.authority_information_access_value:
                for entry in self.authority_information_access_value:
                    if entry['access_method'].native == 'ca_issuers':
                        location = entry['access_location']
                        if location.name != 'uniform_resource_identifier':
                            continue
                        url = location.native
                        if url.lower()[0:7] == 'http://':
                            self._issuer_cert_urls.append(url)
        return self._issuer_cert_urls

    @property
    def delta_crl_distribution_points(self):
        """
        Returns delta CRL URLs - only applies to complete CRLs

        :return:
            A list of zero or more DistributionPoint objects
        """

        if self._delta_crl_distribution_points is None:
            self._delta_crl_distribution_points = []

            if self.freshest_crl_value is not None:
                for distribution_point in self.freshest_crl_value:
                    distribution_point_name = distribution_point['distribution_point']
                    # RFC 5280 indicates conforming CA should not use the relative form
                    if distribution_point_name.name == 'name_relative_to_crl_issuer':
                        continue
                    # This library is currently only concerned with HTTP-based CRLs
                    for general_name in distribution_point_name.chosen:
                        if general_name.name == 'uniform_resource_identifier':
                            self._delta_crl_distribution_points.append(distribution_point)

        return self._delta_crl_distribution_points

    @property
    def signature(self):
        """
        :return:
            A byte string of the signature
        """

        return self['signature'].native

    @property
    def sha1(self):
        """
        :return:
            The SHA1 hash of the DER-encoded bytes of this certificate list
        """

        if self._sha1 is None:
            self._sha1 = hashlib.sha1(self.dump()).digest()
        return self._sha1

    @property
    def sha256(self):
        """
        :return:
            The SHA-256 hash of the DER-encoded bytes of this certificate list
        """

        if self._sha256 is None:
            self._sha256 = hashlib.sha256(self.dump()).digest()
        return self._sha256



import json

import pyscripts

from msgbus import (
    FM_PARAM,
    TK_SYS_ACK,
    TK_PRN_GETSTATUS,
)
from sysactions import (
    action,
    get_model,
    get_cfg,
    print_report,
    get_poslist,
)
from bustoken import TK_PROD_GET_PRINTERS_INFO

from utilfunctions import get_device_service

mb_context = pyscripts.mbcontext


@action
def get_printer_status(pos_id):
    printer_info_list = []

    printer_ok_message = "OK"
    printer_not_ok_message = "Falha"

    msg = mb_context.MB_EasySendMessage("ProductionSystem", TK_PROD_GET_PRINTERS_INFO, FM_PARAM, None)

    if msg.token == TK_SYS_ACK:
        printers = json.loads(msg.data)
        for printer in printers:
            printer_info = [printer]
            try:
                msg = mb_context.MB_EasySendMessage(printer, TK_PRN_GETSTATUS, FM_PARAM, None)

                if msg.data == "0":
                    printer_info.append(printer_ok_message)
                else:
                    printer_info.append(printer_not_ok_message)
            except Exception as _:
                printer_info.append(printer_not_ok_message)
            printer_info_list.append(printer_info)

    pos_list = get_poslist()
    for pos in sorted(pos_list):
        printer_info = ["POS {}".format(str(pos).zfill(2))] if pos != 0 else ["Delivery"]
        try:
            model = get_model(pos)
            pod_function = get_cfg(pos).key_value('pod')
            if pod_function in ("OT", "DL"):
                continue

            printer = get_device_service(
                pos_id=pos_id,
                device_type="printer",
            )
            msg = mb_context.MB_EasySendMessage(printer, TK_PRN_GETSTATUS, FM_PARAM, None)

            if msg.data == "0":
                printer_info.append(printer_ok_message)
            else:
                printer_info.append(printer_not_ok_message)
        except Exception as _:
            printer_info.append(printer_not_ok_message)
        printer_info_list.append(printer_info)

    model = get_model(pos_id)
    print_report(pos_id, model, True, "printer_report", pos_id, printer_info_list)



# encoding: utf-8
from datetime import (
    datetime,
    timedelta
)

from msgbus import MBEasyContext
from typing import Union
from utils import get_pos_list
from reportsdata.tefcancellationreport import (
    TefCancellationReportGenerator,
    TefCancellationReportPresenter
)
from tefcancellationreport.repository import OrderRepository


class ReportGenerator(object):

    @staticmethod
    def generate(
        mb_context,  # type: MBEasyContext
        pos_id,      # type: str
        *args        # type: Union[str, unicode]
    ):
        # type: (...) -> tuple

        pos_list = get_pos_list(mb_context)
        initial_date = args[1]
        end_date = args[2]

        order_repository = OrderRepository(
            mb_context=mb_context,
            pos_list=pos_list
        )
        generator = TefCancellationReportGenerator(
            order_repository=order_repository,
            initial_date=initial_date,
            end_date=end_date
        )
        presenter = TefCancellationReportPresenter(38)

        return generator, presenter



from enum import Enum


class Transfer(object):
    def __init__(self, type, value):
        # type: (Type, float) -> None
        self.type = type
        self.value = value

    class Type(Enum):
        initial_float = 1
        cash_in = 2
        cash_out = 3
        last_cash_out = 4



from abc import ABCMeta, abstractmethod


class OperatorNameRetriever(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_operator_name(self, operator_id):
        # type: (int) -> str
        raise NotImplementedError()



from xml.etree import cElementTree as Etree

from model.log import LogEntry
from _EventProcessor import EventProcessor


def _get_log_entry_from_pos_event(data, event_type):
    event = Etree.fromstring(data)
    event_data = event[0] if "Event" in event.tag else event

    created_at = event_data.get("created_at_gmt")
    session_id = event_data.get("session_id")
    business_period = session_id.split("period=")[1]

    return LogEntry(
        pos_id=event_data.get("pos_id") or -1,
        order_id=event_data.get("order_id"),
        business_period=business_period,
        event_type=event_type,
        created_at_gmt=created_at,
        session_id=session_id,
        event_data=Etree.tostring(event_data)
    )


class ProductionEventProcessor(EventProcessor):
    def parse_data(self, data, event_type):
        return _get_log_entry_from_pos_event(data, event_type)



# -*- coding: utf-8 -*-
import json

import sysactions
from bustoken import TK_REMOTE_ORDER_VOID_REMOTE_ORDER
from custom_sysactions import user_authorized_to_execute_action
from msgbus import TK_SYS_NAK
from systools import sys_log_exception

from .. import mb_context


@sysactions.action
@user_authorized_to_execute_action
def do_void_remote_order(
    pos_id,     # type: str
    remote_id,  # type: str
):
    # type: (...) -> str

    return void_remote_order(
        pos_id=pos_id,
        remote_id=remote_id,
    )


def void_remote_order(
    pos_id,      # type: str
    remote_id,   # type: str
):
    # type: (...) -> str

    try:
        token = TK_REMOTE_ORDER_VOID_REMOTE_ORDER
        data = str(remote_id)
        timeout = 10000 * 1000
        msg = mb_context.MB_EasySendMessage('RemoteOrder', token, data=data, timeout=timeout)

        error_message = None
        if msg.token == TK_SYS_NAK:
            error_message = msg.data
            if not error_message:
                model = sysactions.get_model(pos_id)
                error_message = sysactions.translate_message(model, "UNKNOWN_ERROR")

        return error_message

    except Exception as _:
        sys_log_exception("Error voiding delivery order")
        raise



from datetime import datetime
from datetime import timedelta

from .._compat import string_types
from ..datastructures import CallbackDict
from ..http import dump_age
from ..http import dump_csp_header
from ..http import dump_header
from ..http import dump_options_header
from ..http import http_date
from ..http import parse_age
from ..http import parse_csp_header
from ..http import parse_date
from ..http import parse_options_header
from ..http import parse_set_header
from ..utils import cached_property
from ..utils import environ_property
from ..utils import get_content_type
from ..utils import header_property
from ..wsgi import get_content_length


class CommonRequestDescriptorsMixin(object):
    """A mixin for :class:`BaseRequest` subclasses.  Request objects that
    mix this class in will automatically get descriptors for a couple of
    HTTP headers with automatic type conversion.

    .. versionadded:: 0.5
    """

    content_type = environ_property(
        "CONTENT_TYPE",
        doc="""The Content-Type entity-header field indicates the media
        type of the entity-body sent to the recipient or, in the case of
        the HEAD method, the media type that would have been sent had
        the request been a GET.""",
    )

    @cached_property
    def content_length(self):
        """The Content-Length entity-header field indicates the size of the
        entity-body in bytes or, in the case of the HEAD method, the size of
        the entity-body that would have been sent had the request been a
        GET.
        """
        return get_content_length(self.environ)

    content_encoding = environ_property(
        "HTTP_CONTENT_ENCODING",
        doc="""The Content-Encoding entity-header field is used as a
        modifier to the media-type. When present, its value indicates
        what additional content codings have been applied to the
        entity-body, and thus what decoding mechanisms must be applied
        in order to obtain the media-type referenced by the Content-Type
        header field.

        .. versionadded:: 0.9""",
    )
    content_md5 = environ_property(
        "HTTP_CONTENT_MD5",
        doc="""The Content-MD5 entity-header field, as defined in
        RFC 1864, is an MD5 digest of the entity-body for the purpose of
        providing an end-to-end message integrity check (MIC) of the
        entity-body. (Note: a MIC is good for detecting accidental
        modification of the entity-body in transit, but is not proof
        against malicious attacks.)

        .. versionadded:: 0.9""",
    )
    referrer = environ_property(
        "HTTP_REFERER",
        doc="""The Referer[sic] request-header field allows the client
        to specify, for the server's benefit, the address (URI) of the
        resource from which the Request-URI was obtained (the
        "referrer", although the header field is misspelled).""",
    )
    date = environ_property(
        "HTTP_DATE",
        None,
        parse_date,
        doc="""The Date general-header field represents the date and
        time at which the message was originated, having the same
        semantics as orig-date in RFC 822.""",
    )
    max_forwards = environ_property(
        "HTTP_MAX_FORWARDS",
        None,
        int,
        doc="""The Max-Forwards request-header field provides a
        mechanism with the TRACE and OPTIONS methods to limit the number
        of proxies or gateways that can forward the request to the next
        inbound server.""",
    )

    def _parse_content_type(self):
        if not hasattr(self, "_parsed_content_type"):
            self._parsed_content_type = parse_options_header(
                self.environ.get("CONTENT_TYPE", "")
            )

    @property
    def mimetype(self):
        """Like :attr:`content_type`, but without parameters (eg, without
        charset, type etc.) and always lowercase.  For example if the content
        type is ``text/HTML; charset=utf-8`` the mimetype would be
        ``'text/html'``.
        """
        self._parse_content_type()
        return self._parsed_content_type[0].lower()

    @property
    def mimetype_params(self):
        """The mimetype parameters as dict.  For example if the content
        type is ``text/html; charset=utf-8`` the params would be
        ``{'charset': 'utf-8'}``.
        """
        self._parse_content_type()
        return self._parsed_content_type[1]

    @cached_property
    def pragma(self):
        """The Pragma general-header field is used to include
        implementation-specific directives that might apply to any recipient
        along the request/response chain.  All pragma directives specify
        optional behavior from the viewpoint of the protocol; however, some
        systems MAY require that behavior be consistent with the directives.
        """
        return parse_set_header(self.environ.get("HTTP_PRAGMA", ""))


class CommonResponseDescriptorsMixin(object):
    """A mixin for :class:`BaseResponse` subclasses.  Response objects that
    mix this class in will automatically get descriptors for a couple of
    HTTP headers with automatic type conversion.
    """

    @property
    def mimetype(self):
        """The mimetype (content type without charset etc.)"""
        ct = self.headers.get("content-type")
        if ct:
            return ct.split(";")[0].strip()

    @mimetype.setter
    def mimetype(self, value):
        self.headers["Content-Type"] = get_content_type(value, self.charset)

    @property
    def mimetype_params(self):
        """The mimetype parameters as dict. For example if the
        content type is ``text/html; charset=utf-8`` the params would be
        ``{'charset': 'utf-8'}``.

        .. versionadded:: 0.5
        """

        def on_update(d):
            self.headers["Content-Type"] = dump_options_header(self.mimetype, d)

        d = parse_options_header(self.headers.get("content-type", ""))[1]
        return CallbackDict(d, on_update)

    location = header_property(
        "Location",
        doc="""The Location response-header field is used to redirect
        the recipient to a location other than the Request-URI for
        completion of the request or identification of a new
        resource.""",
    )
    age = header_property(
        "Age",
        None,
        parse_age,
        dump_age,
        doc="""The Age response-header field conveys the sender's
        estimate of the amount of time since the response (or its
        revalidation) was generated at the origin server.

        Age values are non-negative decimal integers, representing time
        in seconds.""",
    )
    content_type = header_property(
        "Content-Type",
        doc="""The Content-Type entity-header field indicates the media
        type of the entity-body sent to the recipient or, in the case of
        the HEAD method, the media type that would have been sent had
        the request been a GET.""",
    )
    content_length = header_property(
        "Content-Length",
        None,
        int,
        str,
        doc="""The Content-Length entity-header field indicates the size
        of the entity-body, in decimal number of OCTETs, sent to the
        recipient or, in the case of the HEAD method, the size of the
        entity-body that would have been sent had the request been a
        GET.""",
    )
    content_location = header_property(
        "Content-Location",
        doc="""The Content-Location entity-header field MAY be used to
        supply the resource location for the entity enclosed in the
        message when that entity is accessible from a location separate
        from the requested resource's URI.""",
    )
    content_encoding = header_property(
        "Content-Encoding",
        doc="""The Content-Encoding entity-header field is used as a
        modifier to the media-type. When present, its value indicates
        what additional content codings have been applied to the
        entity-body, and thus what decoding mechanisms must be applied
        in order to obtain the media-type referenced by the Content-Type
        header field.""",
    )
    content_md5 = header_property(
        "Content-MD5",
        doc="""The Content-MD5 entity-header field, as defined in
        RFC 1864, is an MD5 digest of the entity-body for the purpose of
        providing an end-to-end message integrity check (MIC) of the
        entity-body. (Note: a MIC is good for detecting accidental
        modification of the entity-body in transit, but is not proof
        against malicious attacks.)""",
    )
    content_security_policy = header_property(
        "Content-Security-Policy",
        None,
        parse_csp_header,
        dump_csp_header,
        doc="""The Content-Security-Policy header adds an additional layer of
        security to help detect and mitigate certain types of attacks.""",
    )
    content_security_policy_report_only = header_property(
        "Content-Security-Policy-Report-Only",
        None,
        parse_csp_header,
        dump_csp_header,
        doc="""The Content-Security-Policy-Report-Only header adds a csp policy
        that is not enforced but is reported thereby helping detect
        certain types of attacks.""",
    )
    date = header_property(
        "Date",
        None,
        parse_date,
        http_date,
        doc="""The Date general-header field represents the date and
        time at which the message was originated, having the same
        semantics as orig-date in RFC 822.""",
    )
    expires = header_property(
        "Expires",
        None,
        parse_date,
        http_date,
        doc="""The Expires entity-header field gives the date/time after
        which the response is considered stale. A stale cache entry may
        not normally be returned by a cache.""",
    )
    last_modified = header_property(
        "Last-Modified",
        None,
        parse_date,
        http_date,
        doc="""The Last-Modified entity-header field indicates the date
        and time at which the origin server believes the variant was
        last modified.""",
    )

    @property
    def retry_after(self):
        """The Retry-After response-header field can be used with a
        503 (Service Unavailable) response to indicate how long the
        service is expected to be unavailable to the requesting client.

        Time in seconds until expiration or date.
        """
        value = self.headers.get("retry-after")
        if value is None:
            return
        elif value.isdigit():
            return datetime.utcnow() + timedelta(seconds=int(value))
        return parse_date(value)

    @retry_after.setter
    def retry_after(self, value):
        if value is None:
            if "retry-after" in self.headers:
                del self.headers["retry-after"]
            return
        elif isinstance(value, datetime):
            value = http_date(value)
        else:
            value = str(value)
        self.headers["Retry-After"] = value

    def _set_property(name, doc=None):  # noqa: B902
        def fget(self):
            def on_update(header_set):
                if not header_set and name in self.headers:
                    del self.headers[name]
                elif header_set:
                    self.headers[name] = header_set.to_header()

            return parse_set_header(self.headers.get(name), on_update)

        def fset(self, value):
            if not value:
                del self.headers[name]
            elif isinstance(value, string_types):
                self.headers[name] = value
            else:
                self.headers[name] = dump_header(value)

        return property(fget, fset, doc=doc)

    vary = _set_property(
        "Vary",
        doc="""The Vary field value indicates the set of request-header
        fields that fully determines, while the response is fresh,
        whether a cache is permitted to use the response to reply to a
        subsequent request without revalidation.""",
    )
    content_language = _set_property(
        "Content-Language",
        doc="""The Content-Language entity-header field describes the
        natural language(s) of the intended audience for the enclosed
        entity. Note that this might not be equivalent to all the
        languages used within the entity-body.""",
    )
    allow = _set_property(
        "Allow",
        doc="""The Allow entity-header field lists the set of methods
        supported by the resource identified by the Request-URI. The
        purpose of this field is strictly to inform the recipient of
        valid methods associated with the resource. An Allow header
        field MUST be present in a 405 (Method Not Allowed)
        response.""",
    )

    del _set_property



from __future__ import absolute_import

import datetime
import uuid

from .._compat import text_type
from ..exceptions import BadRequest
from ..utils import detect_utf_encoding

try:
    import simplejson as _json
except ImportError:
    import json as _json


class _JSONModule(object):
    @staticmethod
    def _default(o):
        if isinstance(o, datetime.date):
            return o.isoformat()

        if isinstance(o, uuid.UUID):
            return str(o)

        if hasattr(o, "__html__"):
            return text_type(o.__html__())

        raise TypeError()

    @classmethod
    def dumps(cls, obj, **kw):
        kw.setdefault("separators", (",", ":"))
        kw.setdefault("default", cls._default)
        kw.setdefault("sort_keys", True)
        return _json.dumps(obj, **kw)

    @staticmethod
    def loads(s, **kw):
        if isinstance(s, bytes):
            # Needed for Python < 3.6
            encoding = detect_utf_encoding(s)
            s = s.decode(encoding)

        return _json.loads(s, **kw)


class JSONMixin(object):
    """Mixin to parse :attr:`data` as JSON. Can be mixed in for both
    :class:`~werkzeug.wrappers.Request` and
    :class:`~werkzeug.wrappers.Response` classes.

    If `simplejson`_ is installed it is preferred over Python's built-in
    :mod:`json` module.

    .. _simplejson: https://simplejson.readthedocs.io/en/latest/
    """

    #: A module or other object that has ``dumps`` and ``loads``
    #: functions that match the API of the built-in :mod:`json` module.
    json_module = _JSONModule

    @property
    def json(self):
        """The parsed JSON data if :attr:`mimetype` indicates JSON
        (:mimetype:`application/json`, see :meth:`is_json`).

        Calls :meth:`get_json` with default arguments.
        """
        return self.get_json()

    @property
    def is_json(self):
        """Check if the mimetype indicates JSON data, either
        :mimetype:`application/json` or :mimetype:`application/*+json`.
        """
        mt = self.mimetype
        return (
            mt == "application/json"
            or mt.startswith("application/")
            and mt.endswith("+json")
        )

    def _get_data_for_json(self, cache):
        try:
            return self.get_data(cache=cache)
        except TypeError:
            # Response doesn't have cache param.
            return self.get_data()

    # Cached values for ``(silent=False, silent=True)``. Initialized
    # with sentinel values.
    _cached_json = (Ellipsis, Ellipsis)

    def get_json(self, force=False, silent=False, cache=True):
        """Parse :attr:`data` as JSON.

        If the mimetype does not indicate JSON
        (:mimetype:`application/json`, see :meth:`is_json`), this
        returns ``None``.

        If parsing fails, :meth:`on_json_loading_failed` is called and
        its return value is used as the return value.

        :param force: Ignore the mimetype and always try to parse JSON.
        :param silent: Silence parsing errors and return ``None``
            instead.
        :param cache: Store the parsed JSON to return for subsequent
            calls.
        """
        if cache and self._cached_json[silent] is not Ellipsis:
            return self._cached_json[silent]

        if not (force or self.is_json):
            return None

        data = self._get_data_for_json(cache=cache)

        try:
            rv = self.json_module.loads(data)
        except ValueError as e:
            if silent:
                rv = None

                if cache:
                    normal_rv, _ = self._cached_json
                    self._cached_json = (normal_rv, rv)
            else:
                rv = self.on_json_loading_failed(e)

                if cache:
                    _, silent_rv = self._cached_json
                    self._cached_json = (rv, silent_rv)
        else:
            if cache:
                self._cached_json = (rv, rv)

        return rv

    def on_json_loading_failed(self, e):
        """Called if :meth:`get_json` parsing fails and isn't silenced.
        If this method returns a value, it is used as the return value
        for :meth:`get_json`. The default implementation raises
        :exc:`~werkzeug.exceptions.BadRequest`.
        """
        raise BadRequest("Failed to decode JSON object: {0}".format(e))



from logging import Logger

from interactor.route.order.remove_coupon.dto import RemoveCouponRequest
from model.config import Config
from model.error_type import PosValidationErrorsType
from model.exception import ValidationError
from model.repository import PosRepositoryModel
from model.services import OrderServiceModel
from validation_util import (
    validate_api_key,
    check_if_business_day_is_opened,
    check_if_operator_is_logged
)


class RemoveCouponInteractor(object):

    def __init__(
            self,
            logger,             # type: Logger
            config,             # type: Config
            pos_repository,     # type: PosRepositoryModel
            order_service       # type: OrderServiceModel
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.order_service = order_service

    def execute(
            self,
            request  # type: RemoveCouponRequest
    ):
        # type: (...) -> None

        pos_id = validate_api_key(
                request=request,
                secure_api_key_list=self.config.api_key_mapping
        )
        if not pos_id:
            raise ValidationError(PosValidationErrorsType.PosIdNotFound)

        model = self.pos_repository.get_model(pos_id)
        pos_ot = self.pos_repository.get_pos_ot(model)

        check_if_business_day_is_opened(
                repository=self.pos_repository,
                model=model
        )
        check_if_operator_is_logged(
                repository=self.pos_repository,
                model=model
        )

        coupon_data, coupon_removed = self.pos_repository.remove_coupon(
                pos_id=pos_id,
                coupon_applied_id=request.coupon_applied_id
        )

        if coupon_removed:
            self.logger.info("Coupon successfully removed. CouponData: {}".format(coupon_data))
            order = self.pos_repository.get_current_order(model)
            order_id = order.get('orderId')
            order = self.pos_repository.get_order_details(
                    pos_id=pos_id,
                    order_id=order_id,
                    pos_ot=pos_ot
            )

            if not order:
                raise ValidationError(PosValidationErrorsType.OrderNotFound)

            order = self.order_service.convert_mw_order_to_order_detail(order)

            return order
        else:
            self.logger.info("Error removing coupon. CouponData: {}".format(coupon_data))
            raise ValidationError(PosValidationErrorsType.CannotApplyCoupon, coupon_data)



from abc import (
    ABCMeta,
    abstractmethod
)
from typing import Any


class ProductInfoServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_product_info(
        self,
        language    # type: str
    ):
        # type: (...) -> Any

        raise NotImplementedError



from application.interactor import BurnVoucherInteractor
from application.model import AsyncUuidMessageProcessor
from application.model.configuration import Configurations
from messagebus import DefaultToken, DataType, Message


class BurnVoucherProcessor(AsyncUuidMessageProcessor):
    def __init__(self, configs, interactor):
        # type: (Configurations, BurnVoucherInteractor) -> None

        super(BurnVoucherProcessor, self).__init__()
        self.logger = configs.logger
        self.configs = configs
        self.interactor = interactor

    def parse_data(self, data):
        # type: (str) -> str

        return str(data)

    def call_business(self, benefit_id):
        # type: (str) -> None

        self.interactor.burn_benefit(benefit_id)

    def format_response(self, message_bus, message, event, request, response):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_ACK, DataType.string, response))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))



from _JustificationReportGenerator import JustificationReportGenerator
from _JustificationReportPresenter import JustificationReportPresenter

__all__ = [
    "JustificationReportGenerator",
    "JustificationReportPresenter",
]



show_welcome_message = {}
show_final_message = {}

class LoyaltyInteractor(object):
    def __init__(
        self,
        pos_id,      # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id

    def set_show_customer_welcome_message(
        self,
        show,    # type: bool
    ):
        # type: (...) -> None

        global show_welcome_message

        if self.pos_id not in show_welcome_message:
            show_welcome_message[self.pos_id] = show

    def get_show_customer_welcome_message(
        self,
    ):
        # type: (...) -> bool

        global show_welcome_message

        if self.pos_id in show_welcome_message:
            return show_welcome_message[self.pos_id]

        return False

    def set_show_customer_final_message(
        self,
        show,    # type: bool
    ):
        # type: (...) -> None

        global show_final_message

        if self.pos_id not in show_final_message:
            show_final_message[self.pos_id] = show

    def get_show_customer_final_message(
        self,
    ):
        # type: (...) -> bool

        global show_final_message

        if self.pos_id in show_final_message:
            return show_final_message[self.pos_id]

        return False



# -*- coding: utf-8 -*-
class GLAccount(object):
    def __init__(self, manager="", envelope="", justification=""):
        self.manager = manager
        self.envelope = envelope
        self.justification = justification



import json
import logging

from datetime import datetime
from sysactions import get_model, get_custom, clear_custom

from application.repository import ServicePeriodRepository
from actions import mb_context
from typing import List, Tuple, Union


class ServicePeriodInteractor(object):
    def __init__(self):
        # type: () -> None

        self.logger = logging.getLogger("PosActions")
        self.service_period = ServicePeriodRepository(mb_context)

    def get_services_period(self):
        # type: () -> List

        try:
            result = []
            current_time = datetime.now().time()
            services = self.service_period.get_services_period()
            day_of_week = str(datetime.now().weekday() + 1)
            for service in services:
                start = datetime.strptime(service["startHour"], "%H:%M:%S").time()
                end = datetime.strptime(service["endHour"], "%H:%M:%S").time()
                week_day = service["weekDays"].split(",")
                service["isValid"] = start <= current_time <= end and day_of_week in week_day
                result.append(service)

            if len(result) > 0:
                result = sorted(result, key=lambda k: (k['isValid'] < True, k['name']))

            return result
        except Exception as _:
            self.logger.exception("Error getting service period")
            return []

    def verify_if_service_periods_exist(self, pos_list):
        # type: (Tuple)-> None

        if len(pos_list) == 0:
            return

        all_service_period_id = self._get_service_periods_id()
        for pos_id in pos_list:
            try:
                if len(all_service_period_id) == 0:
                    clear_custom(pos_id, 'SERVICE_PERIOD', True)
                    continue

                current_service_period = json.loads(self._get_pos_current_service_period(int(pos_id)))
                current_service_id = current_service_period.get("servicePeriodId")
                if current_service_id is not None and current_service_id not in all_service_period_id:
                    clear_custom(pos_id, 'SERVICE_PERIOD', True)

            except Exception as _:
                self.logger.exception('Error clear service period pos controller POS: {}'.format(pos_id))

    @staticmethod
    def _get_pos_current_service_period(pos_id):
        # type: (int)-> Union[unicode, str]

        model = get_model(pos_id)
        special_catalog = get_custom(model, 'SERVICE_PERIOD')
        if special_catalog in ['', None]:
            return '{}'

        return special_catalog

    def _get_service_periods_id(self):
        # type: ()-> [int]

        services = self.get_services_period()
        return [int(service["id"]) for service in services] if len(services) > 0 else []



from typing import List, Union  # noqa

from ._I18nReport import I18nReport
from report import Part  # noqa
from report import Report


class SimpleReport(I18nReport):
    def __init__(self, parts, width=40):
        # type: (List[Union[Part, Report]]) -> None
        self.parts = parts
        self.width = width
        self.i18n = None

    def get_parts(self):
        parts = []
        only_parts = self.filter_parts()
        i18n_parts = self.internationalize(only_parts)
        count = 0
        for i in range(0, len(self.parts)):
            if isinstance(self.parts[i], Report):
                parts.extend(self.parts[i].get_parts())
            else:
                parts.append(i18n_parts[count])
                count += 1

        return parts

    def get_width(self):
        return self.width

    def set_i18n(self, i18n):
        super(SimpleReport, self).set_i18n(i18n)
        for part in self.parts:
            if isinstance(part, I18nReport):
                part.set_i18n(i18n)

    def filter_parts(self):
        return filter(lambda x: isinstance(x, Part), self.parts)

    def internationalize(self, parts):
        ret_parts = []
        for part in parts:
            try:
                if part.text is not None:
                    ret_parts.append(Part(self.i18n.internationalize_text(part.text), commands=part.commands))
                else:
                    ret_parts.append(part)
            except UnicodeDecodeError as _:
                ret_parts.append(Part(part.text, commands=part.commands))

        return ret_parts



import base64

from datetime import datetime, timedelta
from time import sleep

from typing import Dict
from xml.etree import cElementTree as eTree


from actions.eft import cancel_eft_transactions
from old_helper import convert_from_utf_to_localtime

import sysactions

from application.interactor import OrderInteractor
from custom_sysactions import user_authorized_to_execute_action
from manager.reports import justification_report
from utilfunctions import get_custom_property_value
from utilconfigurations import must_require_void_reason
from utils import (
    get_void_reason_id,
    void_order,
    void_fiscal_order,
    get_canceled_by,
    get_fixed_void_reason,
)
from .do_void_remote_order import (
    void_remote_order,
)
from .. import logger
VOID_ATTEMPTS = 3


@sysactions.action
@user_authorized_to_execute_action
def do_void_order(
    pos_id,
    fiscal_void='false',
    order_id="",
    show_cancel_message=True,
    is_mobile=False,
    auth_data=None,
    received_void_reason=None
):
    # type: (...) -> bool

    is_fiscal_void = fiscal_void.lower() == 'true'
    is_mobile = str(is_mobile).lower() == 'true'

    try:
        model = sysactions.get_model(pos_id)

        auth_user = auth_data if auth_data is not None else None
        canceled_by = get_canceled_by(
            pos_id,
            model,
            auth_user
        )

        if is_fiscal_void:
            order_id, void_reason = _void_fiscal_order(
                pos_id=pos_id,
                canceled_by=canceled_by
            )
        elif received_void_reason is None:
            void_reason = _void_non_fiscal_order(
                pos_id=pos_id,
                order_id=order_id,
                model=model,
                canceled_by=canceled_by,
            )
        else:
            void_reason = get_fixed_void_reason(
                reason=received_void_reason,
                canceled_by=canceled_by,
            )

        remote_order_id = _void_partner_order(
            pos_id=pos_id,
            order_id=order_id,
        )
        if remote_order_id:
            sysactions.show_messagebox(pos_id, "$REMOTE_ORDER_WILL_BE_VOIDED")
            raise sysactions.StopAction()

        _void_order(
            pos_id=pos_id,
            order_id=order_id,
            void_reason=void_reason,
        )

        _cancel_eft_transactions(
            pos_id=pos_id,
            model=model,
            order_id=order_id,
            is_mobile=is_mobile,
            canceled_by=canceled_by,
        )

        justification_type = 'VOID_PAID_ORDER' if is_fiscal_void else 'VOID_CURRENT_ORDER'
        justification_report(
            pos_id=pos_id,
            justification_type=justification_type,
            justification=void_reason.get('VOID_REASON_DESCR'),
            authorized_by=canceled_by,
            order_id=order_id if order_id not in ["", "None", None] else None
        )

        if show_cancel_message:
            sysactions.show_messagebox(pos_id, "$ORDER_VOIDED_WITH_SUCCESS")

        return True

    except sysactions.StopAction as _:
        return False

    except Exception as _: # noqa
        logger.exception("Error voiding order {}".format(str(order_id) if order_id else 'undefined'))
        if show_cancel_message:
            sysactions.show_messagebox(pos_id, "$ERROR_CANCEL_ORDER")

        return False


def _void_partner_order(
    pos_id,     # type: str
    order_id,   # type: str
):
    # type: (...) -> str

    remote_order_id = get_custom_property_value(
        pos_id=pos_id,
        custom_property="REMOTE_ORDER_ID",
        order_id=order_id,
    )

    if remote_order_id is not None:
        void_remote_order(
            pos_id=pos_id,
            remote_id=remote_order_id,
        )

    return remote_order_id


def _void_order(
    pos_id,         # type: str
    order_id,       # type: str
    void_reason,    # type: Dict
):
    # type: (...) -> None

    for _ in range(VOID_ATTEMPTS):
        try:
            void_order(pos_id, order_id=order_id, void_reason=void_reason)
            break
        except Exception as _: # noqa
            logger.exception("Error voiding order")
            sleep(1)


def _print_tef_receipts(
    pos_id,     # type: str
    is_mobile,  # type: bool
    order_id,   # type: str
):
    # type: (...) -> None

    if not is_mobile:
        from posactions import get_printer_receipt
        printer_receipt = get_printer_receipt()
        printer_receipt.print_tef_receipts(
            pos_id=pos_id,
            cancel_receipt=True,
            order_id=order_id
        )


def _cancel_eft_transactions(
    pos_id,         # type: str
    model,          # type: eTree.XML
    order_id,       # type: str
    is_mobile,      # type: bool
    canceled_by,    # type: str
):
    # type: (...) -> None

    from posactions import is_table_service
    is_table_service = is_table_service(model)

    cancel_eft_transactions(
        pos_id,
        model,
        order_id,
        tender=None,
        is_table_service=is_table_service,
        is_mobile=is_mobile,
        operator_id=canceled_by,
    )

    _print_tef_receipts(
        pos_id=pos_id,
        is_mobile=is_mobile,
        order_id=order_id,
    )


def _void_non_fiscal_order(
        pos_id,             # type: str
        order_id,           # type: str
        model,              # type: eTree.XML
        canceled_by,        # type: str
):
    void_reason = get_fixed_void_reason(
        reason="USER_CANCELLATION",
        canceled_by=canceled_by,
    )

    void_type = "VOID_CURRENT_ORDER"
    if must_require_void_reason(void_type):
        void_reason = get_void_reason_id(
            pos_id=pos_id,
            void_type=void_type,
            authentication_user=canceled_by,
        )
        if not void_reason:
            raise sysactions.StopAction()

    if order_id in ["", None, "None"]:
        sysactions.check_current_order(pos_id, model)

    return void_reason


def _void_fiscal_order(
    pos_id,         # type: str
    canceled_by,    # type: str
):
    # type: (...) -> (str, dict)

    order_id = ""
    limit_datetime = datetime.utcnow() - timedelta(minutes=30)
    date_format = "%Y-%m-%dT%H:%M:%S"

    fiscalized_orders = OrderInteractor(pos_id=pos_id).get_fiscalized_orders(
        from_date=limit_datetime.strftime(date_format),
        pos_list=sysactions.get_poslist(),
    )

    order_options = []
    baseurl = "/mwapp/services/PosController/POS{}/?token=TK_POS_EXECUTEACTION&format=2&isBase64=true&payload="
    baseurl = baseurl.format(pos_id)
    order_descr = "{} - {} POS{:02} - {:05} - R${:.2f}"

    for order in fiscalized_orders:
        descr = order_descr.format(
            convert_from_utf_to_localtime(datetime.strptime(order.fiscalization_date, date_format)).strftime('%H:%M'),
            order.pod_type,
            int(order.originator_id[3:]),
            int(order.order_id),
            float(order.total_gross) + float(order.tip)
        )
        payload = base64.b64encode("\0".join(("doOrderPicture", pos_id, order.order_id)))
        url = baseurl + payload
        order_options.append((order.order_id, descr, url))

    if len(order_options) > 0:
        selected = sysactions.show_order_preview(
            pos_id,
            order_options,
            "$SELECT_AN_ORDER_TO_CANCEL",
            buttons="CANCEL|OK",
            onthefly=True
        )
        if (selected is None) or (selected[0] == "0"):
            raise sysactions.StopAction()

        order_id = selected[1]

    if order_id:
        remote_order_id = _void_partner_order(
            pos_id=pos_id,
            order_id=order_id,
        )
        if remote_order_id:
            sysactions.show_messagebox(pos_id, "$REMOTE_ORDER_WILL_BE_VOIDED")
            raise sysactions.StopAction()

        void_reason = get_fixed_void_reason(
            reason="USER_FISCAL_CANCELLATION",
            canceled_by=canceled_by,
        )

        void_type = "VOID_PAID_ORDER"
        if must_require_void_reason(void_type):
            void_reason = get_void_reason_id(
                pos_id=pos_id,
                void_type=void_type,
                authentication_user=canceled_by
            )
            if not void_reason:
                raise sysactions.StopAction()

        originator_pos_id = int([x for x in fiscalized_orders if x.order_id == order_id][0].originator_id[3:])
        void_fiscal_order(originator_pos_id, order_id)
    else:
        sysactions.show_messagebox(pos_id, "$NO_ORDER_FOUND_TO_CANCEL")
        raise sysactions.StopAction()

    return order_id, void_reason



from _JustificationDiscountBodyDto import JustificationDiscountBodyDto
from _JustificationDiscountGeneratorDto import JustificationDiscountGeneratorDto
from _JustificationDiscountHeaderDto import JustificationDiscountHeaderDto

__all__ = [
    "JustificationDiscountBodyDto",
    "JustificationDiscountGeneratorDto",
    "JustificationDiscountHeaderDto",
]



from logging import Logger

from model.config import Config
from model.electronic_message import ElectronicMessage
from interactor.route.electronic_payment.cache.dto import CacheRequest
from typing import List


class CacheInteractor(object):
    def __init__(self, logger, config, electronic_messages):
        # type: (Logger, Config, List[ElectronicMessage]) -> None

        self.logger = logger
        self.config = config
        self.electronic_messages = electronic_messages

    def execute(self, request):
        # type: (CacheRequest) -> None

        self.electronic_messages.append(request.electronic_message)



# -*- coding: utf-8 -*-
import json

from sysactions import (
    action,
    show_messagebox,
)

from .. import logger
from operator_by_sector import (
    update_operator_sector_info,
    get_sectors,
)

ALL_SECTORS = "$ALL_SECTORS"


@action
def save_sector_operators(
    pos_id,                 # type: str
    available_operators,    # type: str
    selected_operators,     # type: str
    selected_sector,        # type: str
):
    # type: (...) -> None

    try:
        if not selected_sector:
            raise

        available_operators_list = json.loads(available_operators)
        selected_operators_list = json.loads(selected_operators)

        if selected_sector != ALL_SECTORS:
            insert_sector_operators = [[selected_sector, s.get("id")] for s in selected_operators_list]
            delete_sector_operators = [[selected_sector, s.get("id")] for s in available_operators_list]
        else:
            insert_sector_operators = []
            delete_sector_operators = []
            all_sectors = get_sectors(pos_id=pos_id)
            for sector in all_sectors:
                if sector == ALL_SECTORS:
                    continue
                insert_sector_operators.extend([[sector, s.get("id")] for s in selected_operators_list])
                delete_sector_operators.extend([[sector, s.get("id")] for s in available_operators_list])

        update_info = dict(
            delete=delete_sector_operators,
            insert=insert_sector_operators,
        )
        update_operator_sector_info(
            pos_id=pos_id,
            update_info=update_info,
        )

        show_messagebox(
            posid=pos_id,
            message="$OPERATORS_BY_SECTOR_UPDATE_SUCCESS",
        )
    except Exception as e:
        logger.exception('[save_sector_operators] Exception {}'.format(e))
        show_messagebox(
            posid=pos_id,
            message="$ERROR_SAVING_OPERATORS_BY_SECTOR",
            buttons="$EXIT",
        )



from xml.etree import cElementTree as eTree
from posot import OrderTaker


def set_custom_properties(pos_ot, sale_line, seat, sale_type):
    # type: (OrderTaker, eTree.XML, str, str) -> None

    line_number = sale_line.get("lineNumber")
    part_code = sale_line.get("partCode")

    custom_properties = {}
    if seat and seat != '0':
        custom_properties["seat"] = seat
    if sale_type:
        custom_properties["saleType"] = sale_type

    if len(custom_properties) > 0:
        pos_ot.setOrderCustomProperties(custom_properties, "", line_number, "1", "0", part_code)


def update_or_remove_sale_type(pos_ot, sale_line, sale_type):
    # type: (OrderTaker, eTree.XML, str) -> None

    line_number = sale_line.get("lineNumber")
    part_code = sale_line.get("partCode")

    pos_ot.setOrderCustomProperty("saleType", sale_type, "", line_number, "1", "0", part_code)



# encoding: utf-8
from datetime import datetime

from typing import Dict

from tefcancellationreport.repository import OrderRepository


class TefCancellationReport(object):
    def __init__(
        self,
        order_repository    # type: OrderRepository
    ):
        # type: (...) -> None

        self.order_repository = order_repository

    def get_tef_tenders_by_date(
        self,
        initial_date,   # type: datetime
        end_date        # type: datetime
    ):
        # type: (...) -> Dict

        tenders_by_pos = self.order_repository.get_tef_tenders_by_pos(
            initial_date=initial_date,
            end_date=end_date
        )

        tenders_by_date = {}
        for tenders in tenders_by_pos:
            for date, items in tenders.items():
                if date in tenders_by_date:
                    tenders_by_date[date].extend(items)
                else:
                    tenders_by_date[date] = items

        return tenders_by_date

    def get_card_brands(self):
        # type: () -> Dict

        return self.order_repository.get_card_brands()



from abc import (
    ABCMeta,
    abstractmethod
)
from datetime import datetime
from typing import (
    List,
    Optional,
    Tuple,
)


class OrderRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_database_dir(self):
        # type: () -> str

        raise NotImplementedError()

    @abstractmethod
    def get_backup_database_dir(self):
        # type: () -> str

        raise NotImplementedError()

    @abstractmethod
    def get_orders(
        self,
        pos_id,         # type: int
        end_date,       # type: str,
        order_states,   # type: List[str]
    ):
        # type: (...) -> List[Tuple[int, str]]

        raise NotImplementedError()

    @abstractmethod
    def get_business_period_from_oldest_order(self):
        # type: (...) -> Optional[datetime]

        raise NotImplementedError()

    @abstractmethod
    def fill_order_backup_database(
        self,
        pos_id,         # type: int
        suffix_date,    # type: str
        orders          # type: List[int]
    ):
        # type: (...) -> None

        raise NotImplementedError()

    @abstractmethod
    def delete_orders(
        self,
        pos_id,     # type: int
        orders      # type: List[int]
    ):
        # type: (int, List[int]) -> None

        raise NotImplementedError()



from abc import ABCMeta, abstractmethod

from report.command import Command


class CommandProcessor(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_bytes(self, command):
        # type: (Command) -> bytes
        raise NotImplementedError()



from abc import ABCMeta, abstractmethod


class MessageBox(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def show(self, pos_id, message, title='$INFORMATION', icon='info', buttons='$OK', timeout=60000, asynch=False,
             linefeed='\\', focus=True):
        #  type: (str, str, str, str, str, int, bool, str, bool) -> None
        raise NotImplementedError()



from application.domain import TableService, OrderManager  # noqa
from application.domain.ui import ScreenChanger, ScreenIdEnum  # noqa

ACK_RESPONSE = "0"


class CreateTableOrderInteractor(object):

    def __init__(self, table_service, screen_changer, order_manager):
        # type: (TableService, ScreenChanger, OrderManager) -> None
        self.table_service = table_service
        self.screen_changer = screen_changer
        self.order_manager = order_manager

    def execute(self, pos_id, table_id, price_list):
        # type: (str, str, str) -> str

        current_order_state = self.order_manager.get_current_order_state(pos_id)
        if current_order_state == "IN_PROGRESS":
            self.order_manager.store_order(pos_id)

        self.table_service.create_order(pos_id, table_id, price_list)
        return ACK_RESPONSE



from ._TableReportFilterDto import TableReportFilterDto
from ._TableReportDto import TableReportDto
from ._TableReportBodyDto import TableReportBodyDto
from ._TableReportHeaderDto import TableReportHeaderDto
from ._TableReportSaleLineDto import TableReportSaleLineDto
from ._TableReportOrderDto import TableReportOrderDto

__all__ = [
    "TableReportFilterDto",
    "TableReportDto",
    "TableReportBodyDto",
    "TableReportHeaderDto",
    "TableReportSaleLineDto",
    "TableReportOrderDto"
]



from commons.report import Generator
from reports_app.closedaysummary.dto import CloseDaySummaryInfo  # noqa
from reports_app.closedaysummary.dto import CloseDaySummaryDto  # noqa
from reports_app.closedaysummary.dto import CloseDaySummaryHeaderDto  # noqa
from reports_app.closedaysummary.dto import CloseDaySummaryBodyDto
from reports_app.closedaysummary.dto import CashReportBodyLineDto
from reports_app.closedaysummary.dto import OrderInfo


class CloseDayGenerator(Generator):
    def __init__(self, account_repository, order_repository, close_day_summary_info):
        # type: (AccountRepository, OrderRepository, CloseDaySummaryInfo) -> ()
        self.close_day_summary_info = close_day_summary_info
        self.account_repository = account_repository
        self.order_repository = order_repository

    def generate_data(self):
        # type: (CloseDaySummaryInfo) -> CloseDaySummaryDto
        close_day_summary_header_dto = CloseDaySummaryHeaderDto(
            self.close_day_summary_info.pos_id,
            self.close_day_summary_info.store_id,
            self.close_day_summary_info.operator_id,
            self.close_day_summary_info.business_date
        )
        close_day_summary_body_dto = self.generate_body()

        return CloseDaySummaryDto(close_day_summary_header_dto, close_day_summary_body_dto)

    def generate_body(self):
        initial_balance = CashReportBodyLineDto('$INITIAL_BALANCE', None, 0.0)
        sales = CashReportBodyLineDto('$SALES', 0, 0.0)
        voided_orders = CashReportBodyLineDto('$VOIDED_ORDERS', 0, 0.0)
        discounts = CashReportBodyLineDto('$DISCOUNTS', None, None)
        donations = CashReportBodyLineDto('$DONATIONS', 0, 0.0)
        liquid_sales = CashReportBodyLineDto('$LIQUID_SALES', 0, 0.0)
        tender_types = CashReportBodyLineDto('$TENDER_TYPES', None, None)
        cash_out = CashReportBodyLineDto('$CASH_OUT', 0, 0.0)
        cash_in = CashReportBodyLineDto('$CASH_IN', 0, 0.0)
        value_in_drawer = CashReportBodyLineDto('$VALUE_IN_DRAWER', None, 0.0)

        self.parse_account_db(cash_in, cash_out, initial_balance)
        self.parse_order_db(cash_in, cash_out, initial_balance, sales, voided_orders, liquid_sales, donations,
                            value_in_drawer)

        return CloseDaySummaryBodyDto(
            self.close_day_summary_info.authorizer,
            self.close_day_summary_info.open_day_time,
            [initial_balance,
             sales,
             voided_orders,
             discounts,
             donations,
             liquid_sales,
             tender_types,
             cash_out,
             cash_in,
             value_in_drawer])

    def parse_order_db(self, cash_in, cash_out, initial_balance, sales, voided_orders, liquid_sales, donations, value_in_drawer):
        orders_info = self.get_order_db_info()

        for item in orders_info.paid_orders:
            liquid_sales.quantity += 1
            liquid_sales.value += item.total
        for item in orders_info.voided_orders:
            voided_orders.quantity += 1
            voided_orders.value += item.total
        for item in orders_info.paid_orders:
            if item.donation > 0:
                donations.quantity += 1
                donations.value += item.donation
        sales.quantity = liquid_sales.quantity + voided_orders.quantity
        sales.value = liquid_sales.value + voided_orders.value

        value_in_drawer.value = initial_balance.value + cash_in.value - cash_out.value
        for item in orders_info.paid_orders:
            for item_tender in item.tenders:
                if item_tender.type == 0:
                    value_in_drawer.value += item_tender.value
            value_in_drawer.value -= item.change_amount

    def get_order_db_info(self):
        paid_orders = self.order_repository.get_paid_orders_by_real_date(
            self.close_day_summary_info.initial_date,
            self.close_day_summary_info.end_date,
            self.close_day_summary_info.operator_id,
            self.close_day_summary_info.pos_id)
        voided_orders = self.order_repository.get_voided_orders_by_real_date(
            self.close_day_summary_info.initial_date,
            self.close_day_summary_info.end_date,
            self.close_day_summary_info.operator_id,
            self.close_day_summary_info.pos_id)

        return OrderInfo(paid_orders, voided_orders, "")

    def parse_account_db(self, cash_in, cash_out, initial_balance):
        account_info = self.get_account_db_info()

        for item in account_info:
            if item.transfer_cash_in_cash_out == 1 and initial_balance.value == 0.0:
                initial_balance.value = item.transfer_value
            elif item.transfer_cash_in_cash_out == 3 or item.transfer_cash_in_cash_out == 1:
                cash_in.quantity += 1
                cash_in.value += item.transfer_value
            elif item.transfer_cash_in_cash_out == 4:
                cash_out.quantity += 1
                cash_out.value += item.transfer_value

    def get_account_db_info(self):
        account_info = self.account_repository.get_transfers_by_real_date(
            self.close_day_summary_info.initial_date,
            self.close_day_summary_info.end_date,
            self.close_day_summary_info.operator_id,
            self.close_day_summary_info.pos_id,
            'report_by_real_date')
        return account_info



from typing import Optional


class Configs(object):

    def __init__(
        self,
        loader_path=None,           # type: Optional[str]
        component_name=None,        # type: Optional[str]
        required_services=None,     # type: Optional[str]
        instances=None,             # type: Optional[int]
        databases_dir=None,         # type: Optional[str]
        custom_data=None,           # type: Optional[str]
        sector_config=None,         # type: dict
    ):
        # type: (...) -> None

        self.loader_path = loader_path
        self.component_name = component_name
        self.databases_dir = databases_dir
        self.instances = instances
        self.required_services = required_services
        self.custom_data = custom_data
        self.sector_config = sector_config



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional

import simplejson as json
from application.configurator import Configurator
from application.model import EventData
from application.repository import MessageBusRepository


class StoreStatusInteractor(object):

    def __init__(self, configurations):
        # type: (Configurator) -> None

        self.configs = configurations
        self.logger = self.configs.logger

    def build(self, event_data):
        # type: (EventData) -> Optional[str]

        business_period = event_data.data.get("businessPeriod")
        payload_period = datetime.strptime(business_period, "%Y%m%d")
        timestamp = event_data.timestamp + "Z"

        payload = {
            "type": event_data.type,
            "store-code": str(self.configs.store_id),
            "business-dt": payload_period.strftime("%Y-%m-%d"),
            "event-dttm": timestamp
        }

        return json.dumps(payload)



from typing import List

from basereport import ReportColumnDefinition, AlignTypes, TableReport
from report import Part
from report.command import NewLineCommand, AlignCommand, RepeatCommand
from reportsdata.benefitsreport.dto import BenefitsBodyDto
from utils import format_price
from utils.report_formatters import remove_accents


class BenefitConsolidatedBuilder(object):
    def __init__(self, body, columns):
        # type: (BenefitsBodyDto, int) -> ()

        self.body = body
        self.columns = columns

        self.header_table_column_definitions = [
            ReportColumnDefinition(width=15, after_text=' ', fill_with_char=''),
            ReportColumnDefinition(width=5,  after_text=' ', fill_with_char='', align=AlignTypes.CENTER),
            ReportColumnDefinition(width=9,  after_text=' ', fill_with_char='', align=AlignTypes.CENTER),
            ReportColumnDefinition(width=9,  after_text=' ', fill_with_char='', align=AlignTypes.CENTER),
        ]

        self.items_table_column_definitions = [
            ReportColumnDefinition(width=15, after_text=' ', fill_with_char=''),
            ReportColumnDefinition(width=5, after_text=' ', fill_with_char='', align=AlignTypes.CENTER),
            ReportColumnDefinition(width=9, align=AlignTypes.RIGHT, fill_with_char='', after_text=' '),
            ReportColumnDefinition(width=9, align=AlignTypes.RIGHT, fill_with_char=''),
        ]

    def generate(self):
        # type: () -> List

        lines = self.body.items
        parts = []
        parts.extend(self._build_header_table())
        total_qty = 0
        total_default_price = 0.0
        total_discount_value = 0.0
        for line in lines:
            qty, default_value, discount_value, benefit_description = lines[line]
            formatted_value = format_price(default_value)
            formatted_discount = format_price(discount_value)
            benefit_description = str(remove_accents(benefit_description)).upper()
            data_table = [benefit_description, str(qty).zfill(4), formatted_value, formatted_discount]
            parts.extend([TableReport([data_table], self.items_table_column_definitions)])

            total_qty += qty
            total_default_price += default_value
            total_discount_value += discount_value

        total_default_price = format_price(total_default_price)
        total_discount_value = format_price(total_discount_value)
        grand_total = [str(total_qty).zfill(4), total_default_price, total_discount_value]
        parts.extend([Part(None, [RepeatCommand(self.columns, "="), NewLineCommand()])])
        parts.extend(self._build_footer_table(grand_total))
        parts.extend([Part(None, [RepeatCommand(self.columns, "="), NewLineCommand()])])

        return parts

    def _build_header_table(self):
        # type: () -> List

        header = [
            Part(None, [NewLineCommand()]),
            TableReport([["$BENEFIT_REPORT_HEADER_PROMOTION_NAME",
                          "$QTY",
                          "$BENEFIT_REPORT_HEADER_DEFAULT_VALUE",
                          "$BENEFIT_REPORT_HEADER_DISCOUNT_VALUE"]],
                        self.header_table_column_definitions),
        ]
        header.extend([Part(None, [RepeatCommand(self.columns, "="), NewLineCommand()])])
        return header

    def _build_footer_table(self, totals):
        # type: (List) -> List

        return [TableReport([["$TOTAL", totals[0], totals[1], totals[2]]], self.items_table_column_definitions)]



# encoding: utf-8
from datetime import datetime

from systools import sys_log_exception
from typing import List

from basereport import (
    SimpleReport,
    TableReport,
    ReportColumnDefinition,
    AlignTypes,
)
from report import (
    Presenter,
    Part,
)

from report.command import (
    NewLineCommand,
    AlignCommand,
    RepeatCommand,
    FontCommand,
)

from reportsdata.justificationdiscountreport.dto import (
    JustificationDiscountGeneratorDto,
    JustificationDiscountHeaderDto,
    JustificationDiscountBodyDto,
)
from utils.report_formatters import (
    format_line,
    remove_accents,
    format_price,
)


class JustificationDiscountReportPresenter(Presenter):

    def __init__(
        self,
        columns  # type: int
    ):
        # type: (...) -> None

        self.total = 0
        self.columns = columns
        self.title_column_definition = [
            ReportColumnDefinition(
                width=columns,
                align=AlignTypes.CENTER,
                fill_with_char='',
                before_text='',
                after_text='',
            ),
        ]

        self.header_column_definition = [
            ReportColumnDefinition(
                width=13,
                after_fill_text=': ',
                fill_with_char='.',
            ),
            ReportColumnDefinition(
                width=columns - 13,
            ),
        ]

        self.body_column_definition = [
            ReportColumnDefinition(
                width=20,
                after_fill_text='',
                fill_with_char='',
            ),
            ReportColumnDefinition(
                width=columns - 20,
                align=AlignTypes.RIGHT,
            ),
        ]

        self.center_definitions = [
            ReportColumnDefinition(
                width=columns,
            ),
        ]

    def present(
        self,
        dto,     # type: JustificationDiscountGeneratorDto
    ):
        # type: (...) -> SimpleReport

        try:
            parts = [
                Part(
                    commands=[
                        FontCommand(
                            font=FontCommand.Font.B,
                        ),
                    ],
                ),
            ]
            parts.extend(
                self._build_header(
                    header=dto.header,
                ),
            )
            parts.extend(
                self._build_body(
                    body=dto.body,
                ),
            )
            parts.extend(
                [
                    self._new_line(
                        repeat=2,
                    ),
                ],
            )
        except Exception as _:
            sys_log_exception("Error building Benefits Report")
            raise

        return SimpleReport(
            parts=parts,
            width=self.columns,
        )

    def _build_header(
        self,
        header,     # type: JustificationDiscountHeaderDto
    ):
        # type: (...) -> List

        parts = [
            self._get_separator(),
            self._get_title(
                header=header,
            ),
            self._new_line(),
            self._get_date_time(),
            self._get_period(
                header=header,
            ),
            self._get_operator(
                header=header,
            ),
        ]

        return parts

    def _get_title(
        self,
        header,     # type: JustificationDiscountHeaderDto
    ):
        # type: (...) -> str

        title = remove_accents(
           text=header.title,
        )

        return format_line(
            title=None,
            text=title,
            columns=self.columns,
            definition=self.title_column_definition,
        )[0]

    def _build_body(
        self,
        body,   # type: JustificationDiscountBodyDto
    ):
        # type: (...) -> List

        justifications = body.justifications
        if len(justifications) == 0:
            return [self._get_separator()]

        data = []
        for date, items in sorted(justifications.items()):
            data.append(
                self._get_separator(),
            )
            data.append(
                self._build_date_row(
                    date=date,
                ),
            )
            data.extend(
                self._build_rows(
                    items=items,
                ),
            )

        return data

    def _build_date_row(
        self,
        date,   # type: str
    ):
        # type: (...) -> str

        date = datetime.strptime(
            date,
            "%Y-%m-%d"
        ).strftime("%d/%m/%Y")

        return self._get_line(
            title=None,
            text=date,
            definition=self.title_column_definition,
        )[0]

    def _build_rows(
        self,
        items,      # type: List
    ):
        # type: (...) -> List

        parts = []
        size = len(items)

        for item in items:
            size = size - 1
            self._set_row(
                parts=parts,
                title="$ORDER",
                text="$HOUR",
            )
            self._set_row(
                parts=parts,
                title=item.order_id.zfill(8),
                text=item.order_time,
            )
            self._set_row(
                parts=parts,
                title="$TITLE_REPORT_DISCOUNT_COD",
                text="$OPERATOR",
            )
            self._set_row(
                parts=parts,
                title=item.benefit_id,
                text=item.applied_by,
            )
            self._set_row(
                parts=parts,
                title="$DISCOUNT",
                text="$TOTAL",
            )
            self._set_row(
                parts=parts,
                title=format_price(
                    price=float(item.benefit_total_value),
                ),
                text=format_price(
                    price=float(item.order_value),
                ),
            )

            justification = remove_accents(
                text=item.justification,
            )
            self._set_row(
                parts=parts,
                title=None,
                text="$JUSTIFICATION {0}".format(str(justification)),
                definition=self.center_definitions,
            )

            if size > 0:
                parts.extend([
                    Part(
                        text=None,
                        commands=[
                            RepeatCommand(
                                count=self.columns,
                                text="-",
                            ),
                            NewLineCommand(),
                            AlignCommand(
                                alignment=AlignCommand.Alignment.center,
                            ),
                        ],
                    )
                ])

        return parts

    def _set_row(
        self,
        parts,          # type: List
        title,          # type: str
        text,           # type: str
        definition=None,     # type: ReportColumnDefinition
    ):
        # type: (...) -> None

        definition = definition if definition is not None else self.body_column_definition

        parts.extend(
            self._get_line(
                title=title,
                text=text,
                definition=definition,
            )
        )

    def _get_operator(
        self,
        header,   # type: JustificationDiscountHeaderDto
    ):
        # type: (...) -> List

        return self._get_line(
            title="$OPERATOR",
            text="{0} (POS # {1})".format(header.operator, header.pos_id.zfill(2)),
            definition=self.header_column_definition,
        )[0]

    def _get_date_time(
        self,
    ):
        # type: () -> TableReport

        current_datetime = datetime.now().strftime('%d/%m/%y %H:%M:%S')

        return self._get_line(
            title="$DATE_TIME",
            text=current_datetime,
            definition=self.header_column_definition,
        )[0]

    def _get_period(
        self,
        header,     # type: JustificationDiscountHeaderDto
    ):
        # type: (...) -> List

        period = "{0} - {1}".format(
            header.start_date,
            header.end_date,
        )

        return self._get_line(
            title="$PERIOD",
            text=period,
            definition=self.header_column_definition,
        )[0]

    @staticmethod
    def _new_line(
        repeat=1,       # type: int
    ):
        # type: (...) -> Part

        data = Part(
            None,
            [NewLineCommand() for _ in range(0, repeat)],
        )

        return data

    def _get_line(
        self,
        title,              # type: str
        text,               # type: str
        definition,         # type: List[ReportColumnDefinition]
    ):
        # type: (...) -> List[TableReport]

        value = remove_accents(
           text=text,
        )

        return format_line(
            title=title,
            text=value,
            columns=self.columns,
            definition=definition,
        )

    def _get_separator(
        self,
    ):
        # type: (...) -> Part

        return Part(
            text=None,
            commands=[
                RepeatCommand(self.columns, "="),
                NewLineCommand(),
                AlignCommand(AlignCommand.Alignment.center)
            ],
        )




# -*- coding: utf-8 -*-

from msgbus import MBEasyContext
from old_helper import BaseRepository
from persistence import Connection
from typing import List

from dailygoals_services.model import OrderItem
from dailygoals_services.model.repository import ProductRepository as ProductRepositoryRepository


class ProductRepository(
    ProductRepositoryRepository,
    BaseRepository,
):

    def __init__(
        self,
        mb_context,   # type: MBEasyContext
    ):
        # type: (...) -> None

        super(ProductRepository, self).__init__(mb_context)

    def get_order_items_names(
        self,
        order_items,    # type: List[OrderItem]
    ):
        # type: (...) -> List[OrderItem]

        def inner_func(
            conn,    # type: Connection
        ):
            # type: (...) -> List[OrderItem]

            query = self._ProductNamesQuery
            prod_codes_list = map(lambda order: order.pcode, order_items)
            prod_codes_list = ','.join(map(str, prod_codes_list))
            query = query.format(prod_codes_list)

            product_names = [(int(x.get_entry(0)), x.get_entry(1)) for x in conn.select(query)]
            for product_tuple in product_names:
                for order_item in order_items:
                    if product_tuple[0] == order_item.pcode:
                        order_item.name = product_tuple[1]
            return order_items

        return self.execute_with_connection(method=inner_func)

    _ProductNamesQuery = """\
    SELECT *
    FROM Product
    WHERE ProductCode IN ({0})
    """


from typing import List


class SearchCustomerInfoResponse(object):
    def __init__(
            self,
            customer_info,  # type: CustomerInfo
            benefits,       # type: List[Benefit]
     ):
        # type: (...) -> None

        self.customer_info = customer_info

        if benefits:
            self.benefits = benefits
